#!/usr/bin/perl -w

package F90;

use strict;
use FileHandle;

sub what {
  my $code = shift;

  my $what;

  ( $code->[0] =~ m/SUBROUTINE/io ) && ( $what = 'Subroutine' );
  ( $code->[0] =~ m/FUNCTION/io )   && ( $what = 'Function' );
  ( $code->[0] =~ m/MODULE/io )     && ( $what = 'Module' );
  ( $code->[0] =~ m/PROGRAM/io )    && ( $what = 'Program' );

  return $what;

}

sub code {
  my $F90 = shift;

  ( my $fhcode = 'FileHandle'->new( "<$F90" ) )
    or die( "Cannot open <$F90\n" );

 
  my @code = <$fhcode>;

  @code = grep { ! ( m/^\s*!\s*/o or m/^\s*$/o ) } @code;
  
  return \@code;
}


sub use {
  my $code = shift;
  my @code = @$code;
  return [ grep { $_ !~ m/yomlun/io } grep { m/^\s*use\s*(\w+)\b/io; $_ = lc( $1 ) } @code ];
}

sub inc {
  my $code = shift;
  my @code = @$code;
  return [ grep { m/^\s*#include\s*\"(.*\.interface)\"/io; $_ = $1 } @code ];
}

1;



package main;
use strict;
use File::Basename;
use Data::Dumper;

my @obj_ext = qw[ $(OBJDIR)/lapack$(EXT_OBJ) ];
my $ignore = qr/parallel/o;
my $append = "
\$(OBJDIR)/lapack\$(EXT_OBJ): lapack.f
\tcd \$(TMPDIR) && \$(FC77) \$(FFLAGS_RTTOV) \$(FFLAGS_ARCH_lapack) \$(FFLAG_MOD)\$(MODDIR) -c -o \$(OBJDIR)/lapack\$(EXT_OBJ) \$(SRCDIR)/lapack.f

";

my $fhm = 'FileHandle'->new( '>Makefile' );

my @F90 = grep { !/$ignore/o } sort <*.F90>;
my @F90_bin;
my @F90_sub;
my @F90_mod;

my %code;


for my $F90 ( @F90 ) {

  my $code = &F90::code( $F90 );
  $code{$F90} = $code;
  my $what = &F90::what( $code );
  
  if( $what eq 'Program' ) {
    push @F90_bin, $F90;
  } elsif( $what eq 'Module' ) {
    push @F90_mod, $F90;
  } else {
    push @F90_sub, $F90;
  }
}

my %prefix = map( { ( $_, basename( $_, qw( .F90 .f ) ) ) } ( 
    @F90, 'lapack.f', 
    'rttov_parallel_direct.F90', 
    'rttov_parallel_tl.F90',
    'rttov_parallel_ad.F90', 
    'rttov_parallel_k.F90',
  ) 
);

my @bin = map { '$(BINDIR)/' . basename( $_, qw( .F90 ) ) . '$(EXT_EXE)' } @F90_bin;

my @obj_mod = map { '$(OBJDIR)/' . basename( $_, qw( .F90 ) ) . '$(EXT_OBJ)' } @F90_mod;
my @obj_sub = map { '$(OBJDIR)/' . basename( $_, qw( .F90 ) ) . '$(EXT_OBJ)' } @F90_sub;


my ( $len ) = sort { $b <=> $a } map { length( $_ ) } keys( %prefix );
$len += length( "FFLAGS_ARCH_" ) + 2;

for my $f ( sort keys( %prefix ) ) {
  $fhm->printf( "\%-${len}s = \$(FFLAGS_ARCH)\n", "FFLAGS_ARCH_$prefix{$f}" );
}

$fhm->print( "\ninclude ../build/Makefile.inc\n\n" );

$fhm->print( "all: dir mod lib bin\n\n" );

$fhm->print( "
dir:
\t\@mkdir -p \$(BINDIR) \$(OBJDIR) \$(INCDIR) \$(MODDIR) \$(LIBDIR) \$(TMPDIR)

" );

$fhm->print( "mod: @obj_mod\n\n" );

$fhm->print( "bin: @bin\n\n" );


for my $F90 ( @F90_bin ) {
  my $bin = '$(BINDIR)/' . basename( $F90, qw( .F90 ) ) . '$(EXT_EXE)';
  my $inc = &F90::inc( $code{$F90} );
  my $use = &F90::use( $code{$F90} );

# TODO  my @inc = grep { ! ( $_ =~ $ignore ) } map { "../include/$_" } @$inc;

  my @inc = map { "\$(INCDIR)/$_" } @$inc;
  my @obj_use = map { "\$(OBJDIR)/$_\$(EXT_OBJ)" } grep { -f "$_.F90" } @$use;
  my $obj = '$(OBJDIR)/' . basename( $F90, qw( .F90 ) ) . '$(EXT_OBJ)';

  $fhm->print( "
$obj: $F90 @inc @obj_use 
\tcd \$(TMPDIR) && \$(FC) \$(FFLAGS_RTTOV) \$(FFLAGS_ARCH_$prefix{$F90}) \$(FFLAG_MOD)\$(MODDIR) -c \$(SRCDIR)/$F90 -o $obj
" );

  $fhm->print( "
$bin: $obj \$(LIBDIR)/librttov\$(RTTOV_VERSION)\$(EXT_LIB)
\tcd \$(TMPDIR) && \$(FC) -o $bin $obj \$(LDFLAGS) 
" );
}


$fhm->print( "lib: \$(LIBDIR)/librttov\$(RTTOV_VERSION)\$(EXT_LIB) \$(LIBDIR)/librttov\$(RTTOV_VERSION)_parallel\$(EXT_LIB)\n" );

$fhm->print( "
\$(LIBDIR)/librttov\$(RTTOV_VERSION)\$(EXT_LIB): @obj_mod @obj_sub @obj_ext
\t\$(AR) \$(LIBDIR)/librttov\$(RTTOV_VERSION)\$(EXT_LIB) @obj_mod @obj_sub @obj_ext
" );

$fhm->print( "
include ../build/Makefile_parallel
" );


for my $F90 ( @F90_sub ) {

  my $obj = '$(OBJDIR)/' . basename( $F90, qw( .F90 ) ) . '$(EXT_OBJ)';
  
  my $inc = &F90::inc( $code{$F90} );
  my $use = &F90::use( $code{$F90} );
  
#  my @inc = grep { ! ( $_ =~ $ignore ) } map { "../include/$_" } @$inc;


  my @inc = map { "\$(INCDIR)/$_" } @$inc;
  my @obj_use = map { "\$(OBJDIR)/$_\$(EXT_OBJ)" } grep { -f "$_.F90" } @$use;
  
  $fhm->print( "
$obj: $F90 @inc @obj_use
\tcd \$(TMPDIR) && \$(FC) \$(FFLAGS_RTTOV) \$(FFLAGS_ARCH_$prefix{$F90}) \$(FFLAG_MOD)\$(MODDIR) -c -o $obj \$(SRCDIR)/$F90
" );
}

for my $F90 ( @F90_mod ) {

  my $mod = basename( $F90, qw( .F90 ) ) . '$(EXT_MOD)';
  my $obj = '$(OBJDIR)/' . basename( $F90, qw( .F90 ) ) . '$(EXT_OBJ)';
  
  my $use = &F90::use( $code{$F90} );
  
  my @obj_use = map { "\$(OBJDIR)/$_\$(EXT_OBJ)" } grep { -f "$_.F90" } @$use;
  
  $fhm->print( "
$obj: $F90 @obj_use
\tcd \$(TMPDIR) && \$(FC) \$(FFLAGS_RTTOV) \$(FFLAGS_ARCH_$prefix{$F90}) \$(FFLAG_MOD)\$(MODDIR) -c -o $obj \$(SRCDIR)/$F90
\t\@cd \$(TMPDIR) && \$(MVDMOD) $mod \$(MODDIR)/$mod
" );
}

for my $F90 ( @F90_sub ) {

  my $int = '$(INCDIR)/' . basename( $F90, qw( .F90 ) ) . '.interface';
  
  
  $fhm->print( "
$int: $F90 
\t\@\$(MKINTF) $F90 $int
" );
}



$fhm->print( "
clean:
\t\\rm -f \$(OBJDIR)/*\$(EXT_OBJ) \$(LIBDIR)/*\$(EXT_LIB) \$(MODDIR)/*\$(EXT_MOD) \$(BINDIR)/*\$(EXT_EXE) \$(INCDIR)/* \$(TMPDIR)/*

dist: 
\tcd .. ; tar cf - src build rttov_test/rttov_test.pl rttov_test/tests.0 rttov_test/*.sh rttov_test/*.2 | gzip > rttov_dist.tgz
" );


$fhm->print( $append );

$fhm->close();


