#!/bin/ksh
#
# RTTOV_SCATT tests control script
#
# This test requires the (~1GB) Mie Tables to be available
# and placed in a directory pointed to by DATAMIE
# 
# 13/12/2007   Alan Geer   First version
#
######## Edit this section for your pathnames#######
DATAIN=../data    # NB hardcoded into rttovscatt_test.F90
DATART=../rtcoef_rttov7
DATAMIE=../rtcoef_scatt
EXEC=../bin/rttovscatt_test.exe
REF_TEST=../reftest_rttovscatt
TEST=../test_rttovscatt
###################################################
set -x

if [ ! -d ${TEST} ]; then
  mkdir ${TEST}
fi

cwd=`pwd`
cd $TEST

for TEST_NUMBER in 01 02 
do
   
#  export PROGINF=DETAIL
#  export DR_HOOK=1
#  export DR_HOOK_OPT='hpmprof'
#  export OMP_NUM_THREADS=1

  case ${TEST_NUMBER} in
  
    '01') PLATFORM=dmsp ; SATID=13 ; INST=ssmi
   
    cat  > rttovscatt_test_input  <<EOF
2   # Satellite series
13  # Satellite ID
6   # Satellite sensor
57  # zenith angle (degrees)
EOF

    ;;

    '02') PLATFORM=noaa ; SATID=16 ; INST=amsua

    cat  > rttovscatt_test_input  <<EOF
1   # Satellite series
16  # Satellite ID
3   # Satellite sensor
23  # zenith angle (degrees)
EOF
   
    ;;    
  esac
  
  COEF_NAME=rtcoef_${PLATFORM}_${SATID}_${INST}.dat
  rm -f ${COEF_NAME}
  if [ -s  $DATART/${COEF_NAME} ] ; then
    ln -s $DATART/${COEF_NAME} ${COEF_NAME}
  else
    echo "RTCOEF tables not found"
    cd $cwd
    exit
  fi

  MIETABLE_NAME=mietable_${PLATFORM}_${INST}.dat
  rm -f ${MIETABLE_NAME}
  if [ -s  $DATAMIE/${MIETABLE_NAME} ] ; then
    ln -s $DATAMIE/${MIETABLE_NAME} ${MIETABLE_NAME}
  else
    echo "Mie tables not found"
    cd $cwd
    exit
  fi

  $EXEC < rttovscatt_test_input

#  mv drhook.prof.1      drhook.prof.${TEST_NUMBER}.rttov9_scatt
  mv outputscatt.ascii  output.${TEST_NUMBER}.rttov9_scatt

  diff -biw output.${TEST_NUMBER}.rttov9_scatt \
    $REF_TEST/output.${TEST_NUMBER}.rttov9_scatt > diff.${TEST_NUMBER}
  
done

cd $cwd

exit
