!
Subroutine rttov_alloc_prof (errorstatus,nprof,profiles,nlevels, &
  & coef_scatt_ir,asw,addaerosl,addclouds,init,blob)
  ! Description:
  ! allocation/deallocation of a profile structure
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2007, EUMETSAT, All Rights Reserved.
  !
  ! Method:
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       01/12/2002  New F90 code 
  !  1.1       11/10/2007  Add addclouds, addaerosl, init logicals
  !                        nullify unused pointers P.Marguinaud
  !  1.2       26/09/2008  Disable vectorization of the pointer association
  !                        loop; this is for the NEC under -Chopt
  !                        P. Marguinaud
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  Use rttov_const, Only :     &
         & ncldtyp       ,     &
         & errorstatus_info    ,&
         & errorstatus_success ,&
         & errorstatus_fatal   ,&
         & errorstatus_warning 
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only :       &
        & rttov_coef_scatt_ir , &
        & profile_Type,         &
        & blob_type

  Use parkind1, Only : jpim     ,jprb
  Implicit None


  ! subroutine arguments
  ! scalar arguments with intent(out):
  Integer(Kind=jpim),          Intent(out)    :: errorstatus          ! return code
  Integer(Kind=jpim),          Intent(in)     :: nprof                ! number of profiles
  Integer(Kind=jpim),          Intent(in)     :: nlevels              ! number of levels
  Type( profile_Type ),        Intent(inout)  :: profiles(nprof)      ! profiles
  Type( rttov_coef_scatt_ir ), Intent(in)     :: coef_scatt_ir
  Integer(Kind=jpim),          Intent(in)     :: asw                  ! 1=allocate, 0=deallocate
  Logical,                     Intent(in),    optional :: addclouds
  Logical,                     Intent(in),    optional :: addaerosl
  Logical,                     Intent(in),    optional :: init
  Type(blob_type),             intent(inout), optional :: blob

!INTF

#include "rttov_errorreport.interface"


  ! Local Arrays and Scalars:
  Integer(Kind=jpim), Parameter :: n_ios = 100
  Integer(Kind=jpim)            :: alloc_status(n_ios)
  Integer(Kind=jpim)            :: iaer , icld
  Integer(Kind=jpim)            :: j
  Character (len=80) :: errMessage
  Character (len=18) :: NameOfRoutine = 'rttov_alloc_prof'
  REAL(KIND=JPRB) :: ZHOOK_HANDLE
  Logical :: addclouds1
  Logical :: addaerosl1
  Logical :: init1

  !- End of header --------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_ALLOC_PROF',0_jpim,ZHOOK_HANDLE)

  addclouds1 = .false.
  addaerosl1 = .false.
  init1 = .false.
  If( Present( addclouds ) ) addclouds1 = addclouds
  If( Present( addaerosl ) ) addaerosl1 = addaerosl
  If( Present( init ) ) init1 = init

  

  alloc_status(:) = 0
  errorstatus = errorstatus_success
  !  Allocate section
  iaer = max(1_jpim,coef_scatt_ir % fmv_aer_comp)
  icld = max (1_jpim,coef_scatt_ir % fmv_wcl_comp+1_jpim)


  If( asw .eq. 1 ) Then
     Do j = 1, nprof
       nullify( profiles(j) % p )
       nullify( profiles(j) % t )
       nullify( profiles(j) % q )
       nullify( profiles(j) % o3 )
       nullify( profiles(j) % co2 )
       nullify( profiles(j) % n2o )
       nullify( profiles(j) % co )
       nullify( profiles(j) % ch4 )
       nullify( profiles(j) % clw )
       nullify( profiles(j) % aerosols )
       nullify( profiles(j) % cloud )
       nullify( profiles(j) % cfrac )
    EndDo
  EndIf


  If( Present( blob ) .and. ( asw .eq. 1 ) ) Then
  
#include "nullify_blob.h"
  
    allocate( blob % rlvp( nlevels, 9, nprof ), stat = alloc_status(1) )
    
    If( addaerosl1 ) &
      allocate( blob % ralp( iaer, nlevels, nprof ), stat = alloc_status(2) )
    
    If( addclouds1 ) Then
      allocate( blob % rclp( icld, nlevels, nprof ), stat = alloc_status(3) )
      allocate( blob % rtlp( ncldtyp, nlevels, nprof ), stat = alloc_status(4) )
    EndIf

!CDIR NOVECTOR
    Do j = 1, nprof
      profiles(j) % nlevels = nlevels
      profiles(j) % p   => blob % rlvp( :, 1, j )
      profiles(j) % t   => blob % rlvp( :, 2, j )
      profiles(j) % q   => blob % rlvp( :, 3, j )
      profiles(j) % co2 => blob % rlvp( :, 4, j )
      profiles(j) % o3  => blob % rlvp( :, 5, j )
      profiles(j) % n2o => blob % rlvp( :, 6, j )
      profiles(j) % co  => blob % rlvp( :, 7, j )
      profiles(j) % ch4 => blob % rlvp( :, 8, j )
      profiles(j) % clw => blob % rlvp( :, 9, j )
      If( addaerosl1 ) &
        profiles(j) % aerosols => blob % ralp( :, :, j )
      If( addclouds1 ) Then
        profiles(j) % cloud    => blob % rtlp( :, :, j )
        profiles(j) % cfrac    => blob % rclp( :, :, j )
      EndIf
    EndDo

    
    If( any( alloc_status .ne. 0 ) ) Then
      errorstatus = errorstatus_fatal
      Write( errMessage, '( "mem allocation error")' )
      Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_ALLOC_PROF',1_jpim,ZHOOK_HANDLE)
      Return
    EndIf
  EndIf 
   
  If( Present( blob ) .and. ( asw .eq. 0 ) ) Then
   
    
#include "deallocate_blob.h"    
#include "nullify_blob.h"

    If( any( alloc_status .ne. 0 ) ) Then
      errorstatus = errorstatus_fatal
      Write( errMessage, '( "mem deallocation error")' )
      Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
      IF (LHOOK) CALL DR_HOOK('RTTOV_ALLOC_PROF',1_jpim,ZHOOK_HANDLE)
      Return
    EndIf
    
    
  EndIf
  
  
  
  If( ( .not. Present( blob ) ) .and. ( asw .eq. 1 ) ) Then

    do j = 1, nprof
       profiles(j) % nlevels = nlevels

       allocate( profiles(j) % p  ( nlevels ) ,stat= alloc_status(2))
       allocate( profiles(j) % t  ( nlevels ) ,stat= alloc_status(3))
       allocate( profiles(j) % q  ( nlevels ) ,stat= alloc_status(4))
       allocate( profiles(j) % co2( nlevels ) ,stat= alloc_status(5))
       allocate( profiles(j) % o3 ( nlevels ) ,stat= alloc_status(6))
       allocate( profiles(j) % n2o( nlevels ) ,stat= alloc_status(7))
       allocate( profiles(j) % co ( nlevels ) ,stat= alloc_status(8))
       allocate( profiles(j) % ch4( nlevels ) ,stat= alloc_status(9))
       allocate( profiles(j) % clw( nlevels ) ,stat= alloc_status(10))
       If( addaerosl1 ) &
         allocate( profiles(j) % aerosols(iaer,nlevels ) ,stat= alloc_status(11))

       If( addclouds1 ) Then
         allocate( profiles(j) % cloud(icld,nlevels ) ,stat= alloc_status(12))
         allocate( profiles(j) % cfrac(ncldtyp,nlevels ) ,stat= alloc_status(13))
       EndIf
       If( any(alloc_status /= 0) ) then
          errorstatus = errorstatus_fatal
          Write( errMessage, '( "mem allocation error")' )
          Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
          IF (LHOOK) CALL DR_HOOK('RTTOV_ALLOC_PROF',1_jpim,ZHOOK_HANDLE)
          Return
       End If
    end do
       
  EndIf    

 
  
  If( ( .not. Present( blob ) ) .and. ( asw .eq. 0 ) ) Then
    
   
    do j = 1, nprof
       ! deallocate model profiles atmospheric arrays
       deallocate( profiles(j) % p   ,stat=alloc_status(1))
       deallocate( profiles(j) % t   ,stat=alloc_status(2))
       deallocate( profiles(j) % q   ,stat=alloc_status(3))
       deallocate( profiles(j) % o3  ,stat=alloc_status(4))
       deallocate( profiles(j) % co2 ,stat=alloc_status(5))
       deallocate( profiles(j) % n2o ,stat=alloc_status(6))
       deallocate( profiles(j) % co  ,stat=alloc_status(7))
       deallocate( profiles(j) % ch4 ,stat=alloc_status(8))
       deallocate( profiles(j) % clw ,stat=alloc_status(9))
       If( associated( profiles(j) % aerosols ) ) &
         deallocate( profiles(j) % aerosols, stat=alloc_status(10))
       If( associated( profiles(j) % cloud ) ) &
         deallocate( profiles(j) % cloud, stat=alloc_status(11))
       If( associated( profiles(j) % cfrac ) ) &
         deallocate( profiles(j) % cfrac, stat=alloc_status(12))
   
       nullify( profiles(j) % p )
       nullify( profiles(j) % t )
       nullify( profiles(j) % q )
       nullify( profiles(j) % o3 )
       nullify( profiles(j) % co2 )
       nullify( profiles(j) % n2o )
       nullify( profiles(j) % co )
       nullify( profiles(j) % ch4 )
       nullify( profiles(j) % clw )
       nullify( profiles(j) % aerosols )
       nullify( profiles(j) % cloud )
       nullify( profiles(j) % cfrac )
   
   
       If( any(alloc_status /= 0) ) then
          errorstatus = errorstatus_fatal
          Write( errMessage, '( "mem deallocation error")' )
          Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
          IF (LHOOK) CALL DR_HOOK('RTTOV_ALLOC_PROF',1_jpim,ZHOOK_HANDLE)
          Return
       End If
   
    end do
   
   EndIf

  If( init1 .and. ( asw .eq. 1 ) ) Then
    Do j = 1, nprof
      profiles( j ) % p           = 0._jprb
      profiles( j ) % t           = 0._jprb
      profiles( j ) % q           = 0._jprb
      profiles( j ) % co2         = 0._jprb
      profiles( j ) % o3          = 0._jprb
      profiles( j ) % n2o         = 0._jprb
      profiles( j ) % co          = 0._jprb
      profiles( j ) % ch4         = 0._jprb
      profiles( j ) % clw         = 0._jprb
      If( associated( profiles( j ) % aerosols ) ) profiles( j ) % aerosols = 0._jprb
      If( associated( profiles( j ) % cloud )    ) profiles( j ) % cloud    = 0._jprb
      If( associated( profiles( j ) % cfrac )    ) profiles( j ) % cfrac    = 0._jprb
      profiles( j ) % zenangle         = 0._jprb
      profiles( j ) % azangle          = 0._jprb
      profiles( j ) % sunzenangle      = 0._jprb
      profiles( j ) % sunazangle       = 0._jprb
      profiles( j ) % elevation        = 0._jprb
      profiles( j ) % latitude         = 0._jprb
      profiles( j ) % ctp              = 0._jprb
      profiles( j ) % cfraction        = 0._jprb
      profiles( j ) % s2m % t          = 0._jprb 
      profiles( j ) % s2m % q          = 0._jprb       
      profiles( j ) % s2m % o          = 0._jprb
      profiles( j ) % s2m % p          = 0._jprb 
      profiles( j ) % s2m % u          = 0._jprb 
      profiles( j ) % s2m % v          = 0._jprb 
      profiles( j ) % s2m % wfetc      = 0._jprb
      profiles( j ) % skin % t         = 0._jprb  
      profiles( j ) % skin % fastem    = 0._jprb
      profiles( j ) % skin % surftype  = 0._jprb
      profiles( j ) % skin % watertype = 0._jprb
      profiles( j ) % be               = 0._jprb
      profiles( j ) % cosbk            = 0._jprb
      profiles( j ) % ozone_Data   = .False.  
      profiles( j ) % co2_Data     = .False.  
      profiles( j ) % n2o_Data     = .False.  
      profiles( j ) % co_Data      = .False.  
      profiles( j ) % ch4_Data     = .False.  
      profiles( j ) % clw_Data     = .False.  
      profiles( j ) % cld_data     = addclouds1
      profiles( j ) % aer_data     = addaerosl1
      profiles( j ) % idg          = -1_jpim
      profiles( j ) % ish          = -1_jpim
      profiles( j ) % addsolar     = .False.
      profiles( j ) % addrefrac    = .False.
      profiles( j ) % addaerosl    = addaerosl1
      profiles( j ) % addwatrcl    = .False.
    EndDo
  EndIf

  If( asw .eq. 0 ) Then 
    Do j = 1, nprof
      nullify( profiles(j) % p )
      nullify( profiles(j) % t )
      nullify( profiles(j) % q )
      nullify( profiles(j) % o3 )
      nullify( profiles(j) % co2 )
      nullify( profiles(j) % n2o )
      nullify( profiles(j) % co )
      nullify( profiles(j) % ch4 )
      nullify( profiles(j) % clw )
      nullify( profiles(j) % aerosols )
      nullify( profiles(j) % cloud )
      nullify( profiles(j) % cfrac )
    EndDo
  EndIf
 
  IF (LHOOK) CALL DR_HOOK('RTTOV_ALLOC_PROF',1_jpim,ZHOOK_HANDLE)

  Return

End Subroutine rttov_alloc_prof
