Module Rttov_GetOptions

Use parkind1, only: jpim, jprb

Use rttov_unix_env, Only: rttov_iargc, rttov_getarg


Interface GetOption
  Module Procedure GetOptionS, GetOptionSL, &
                   GetOptionI, GetOptionIL, &
                   GetOptionR, GetOptionRL, &
                   GetOptionB
End Interface

Type rttov_option
  Type(rttov_option), Pointer :: next => NULL()
  Character*256 :: key, use
  Logical :: mnd, found
End Type

Public :: GetOption, rttov_option

Private

Contains


Subroutine Check_mnd( key, mnd, use )
 Character*(*) :: key
 Character*(*), optional :: use
 Logical, optional :: mnd
 Character*256 :: prog

 If( Present( mnd ) ) Then
   If( mnd ) Then
     Call rttov_getarg( 0, prog )
     Write( *, '("PROGRAM: ",(a))' ) Trim( prog )
     Write( *, '("ERROR:   Option `",(a),"'' is mandatory")' ) Trim( key )
     If( Present( use ) ) Write( *, '("         ",(a)," : ",(a))' ) Trim( key ), Trim( use )
     Stop 1
   EndIf
 EndIf

End Subroutine

Subroutine FindArgIndex( key, i, n )
 Character*(*) :: key
 Integer(Kind=jpim) :: i, n
 Character*256 :: arg

 n = rttov_iargc()
 Do i = 1, n
   Call rttov_getarg( i, arg )
   If( Trim( arg ) .eq. Trim( key ) ) Return
 EndDo
 i = -1_jpim
End Subroutine

Subroutine FindNextArgIndex( i, j )
 Integer(Kind=jpim) :: i, j, n
 Character*256 :: arg

 n = rttov_iargc()
 Do j = i+1, n
  Call rttov_getarg( j, arg )
  If( arg(1:2) .eq. '--' ) Exit
 EndDo

End Subroutine

Subroutine GetOptionS( key, val, mnd, use )
!
 Character*(*) :: key, val
 Logical, optional :: mnd
 Character*(*), optional :: use
!
 Integer(Kind=jpim) :: i, n
 Character*256 :: arg
 
 Call FindArgIndex( key, i, n )

 If( ( 0 .lt. i ) .and. ( i .lt. n ) ) Then
   Call rttov_getarg( i+1, val )
 Else
   Call check_mnd( key, mnd, use )
 EndIf

End Subroutine

Subroutine GetOptionI( key, val, mnd, use )
!
 Character*(*) :: key
 Integer(Kind=jpim) :: val
 Logical, optional :: mnd
 Character*(*), optional :: use
!
 Integer(Kind=jpim) :: i, n
 Character*256 :: sval
 
 sval = ""
 Call GetOptionS( key, sval, mnd, use )
 If( Trim( sval ) .ne. "" ) Then
   Read( sval, * ) val
 EndIf

End Subroutine

Subroutine GetOptionR( key, val, mnd, use )
!
 Character*(*) :: key
 Real(Kind=jprb) :: val
 Logical, optional :: mnd
 Character*(*), optional :: use
!
 Integer(Kind=jpim) :: i, n
 Character*256 :: sval
 
 sval = ""
 Call GetOptionS( key, sval, mnd, use )
 If( Trim( sval ) .ne. "" ) Then
   Read( sval, * ) val
 EndIf

End Subroutine

Subroutine GetOptionSL( key, val, mnd, use )
!
 Character*(*) :: key
 Character(len=*), Pointer :: val(:)
 Logical, optional :: mnd
 Character*(*), optional :: use
!
 Integer(Kind=jpim) :: i, j, k, n
 Character*256 :: arg
 
 Call FindArgIndex( key, i, n )
 If( i < 0 ) Then
   Call check_mnd( key, mnd, use )
   Return
 EndIf

 Call FindNextArgIndex( i, j )
 Allocate( val( j - i - 1 ) )
 
 Do k = i+1, j-1
   Call rttov_getarg( k, arg )
   val(k-i) = arg
 EndDo

End Subroutine

Subroutine GetOptionIL( key, val, mnd, use )
!
 Character*(*) :: key
 Integer(Kind=jpim), Pointer :: val(:)
 Logical, optional :: mnd
 Character*(*), optional :: use
!
 Character*256, Pointer :: sval(:) => NULL()
 Integer(kind=jpim) :: i, n

 Call GetOptionSL( key, sval, mnd, use )

 If( .Not. Associated( sval ) ) Return

 n = Size( sval )
 Allocate( val( n ) )
 Do i = 1, n
   Read( sval( i ), * ) val( i )
 EndDo

 Deallocate( sval )

End Subroutine

Subroutine GetOptionRL( key, val, mnd, use )
!
 Character*(*) :: key
 Real(Kind=jprb), Pointer :: val(:)
 Logical, optional :: mnd
 Character*(*), optional :: use
!
 Character*256, Pointer :: sval(:) => NULL()
 Integer(kind=jpim) :: i, n

 Call GetOptionSL( key, sval, mnd, use )

 If( .Not. Associated( sval ) ) Return

 n = Size( sval )
 Allocate( val( n ) )
 Do i = 1, n
   Read( sval( i ), * ) val( i )
 EndDo

 Deallocate( sval )

End Subroutine

Subroutine GetOptionB( key, val, use )
!
 Character*(*) :: key
 Logical :: val
 Character*(*), optional :: use
!

 Call FindArgIndex( key, i, n )
 val = i > 0

End Subroutine

End Module
