!
Subroutine rttov_k_bf( &
      & errorstatus,    &
      & nprofiles,      &
      & nchannels,      &
      & channels,       &
      & lprofiles,      &
      & addinterp,      &
      & profiles,       &
      & profiles_k,     &
      & coef,           &
      & coef_scatt_ir,  &
      & optp,           &
      & addsun,         &
      & addaerosl,      &
      & addclouds,      &
      & switchrad,      &
      & calcemis,       &
      & emissivity,     &
      & emissivity_k,   &
      & transmission,   &
      & transmission_k, &
      & radiancedata,   &
      & radiance_k,     &
      & temp, temp_k     ) 

  Use rttov_const, Only :      &
        & errorstatus_success ,&
        & errorstatus_warning ,&
        & errorstatus_fatal   ,&
        & max_optical_depth   ,&
        & sensor_id_mw        ,&
        & sensor_id_ir        ,&
        & sensor_id_po        ,&
        & sensor_id_hi        ,& 
        & ncldtyp             ,&
        & ngases_max          ,&
        & apply_reg_limits

  USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  ! Imported Type Definitions:
  Use rttov_types, Only :             &
        & rttov_coef,                 &
        & rttov_coef_scatt_ir,        &
        & transmission_scatt_ir_type, &
        & rttov_optpar_ir,            &
        & profile_Type,               &
        & transmission_Type,          &
        & radiance_Type,              &
        & rttov_temp

  Use rttov_global, Only: &
        & verbose_checkinput_warnings

  Use parkind1, Only : jpim     ,jprb
  Use rttov_chain
  Implicit None


  !subroutine arguments:
  Integer(Kind=jpim),        Intent(in)    :: nprofiles
  Integer(Kind=jpim),        Intent(in)    :: nchannels
  Integer(Kind=jpim),        Intent(in)    :: channels(nchannels)
  Integer(Kind=jpim),        Intent(in)    :: lprofiles(nchannels)
  Logical,                   Intent(in)    :: addsun
  Logical,                   Intent(in)    :: addaerosl
  Logical,                   Intent(in)    :: addclouds
  Logical,                   Intent(in)    :: addinterp  ! switch for User-Coef-User interpolations
  Logical,                   Intent(in)    :: switchrad  ! true if input is BT

  Type(profile_Type),        Intent(in),    Target :: profiles(nprofiles)
  Type(profile_Type),        Intent(inout), Target :: profiles_k(nchannels)
                                                           ! for calls from RTTOV_CLD_K and RTTOV_SCATT_K.

  Type(rttov_coef),          Intent(in)    :: coef
  Type(rttov_coef_scatt_ir), Intent(in)    :: coef_scatt_ir
  Type(rttov_optpar_ir),     Intent(in)    :: optp

  Logical,                   Intent(in)    :: calcemis(nchannels)

  Integer(Kind=jpim),        Intent(out)   :: errorstatus(nprofiles)

  Real(Kind=jprb),           Intent(inout) :: emissivity(nchannels)
  Real(Kind=jprb),           Intent(inout) :: emissivity_k(nchannels)

  Type(transmission_Type),   Intent(inout) :: transmission! in because of meme allocation
  Type(transmission_Type),   Intent(inout) :: transmission_k ! in because of meme allocation

  Type(radiance_Type),       Intent(inout) :: radiancedata! in because of meme allocation
  Type(radiance_Type),       Intent(inout), optional :: radiance_k
  Type(rttov_temp),          Intent(inout), optional :: temp, temp_k

!INTF


#include "rttov_direct.interface"
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_make_profile_inc.interface"

  
  Type(profile_Type), Target       :: profiles_bf(nprofiles)
  Type(profile_Type), Target       :: profiles_inc(nprofiles)
  Real(Kind=jprb)          :: emissivity_bf(nchannels)
  Logical                  :: calcemis_bf(nchannels)
  Type(transmission_Type)  :: transmission_bf
  Type(radiance_Type)      :: radiancedata_bf
  Integer(Kind=jpim)       :: allocstatus
  Type(chain), Target      :: chain_profiles(nprofiles)
  Type(pchain)             :: p_chain_profiles(nprofiles)
  Type(chain), Target      :: chain_profiles_inc(nprofiles)
  Type(pchain)             :: p_chain_profiles_inc(nprofiles)
  Type(chain), Target      :: chain_profiles_bf(nprofiles)
  Type(pchain)             :: p_chain_profiles_bf(nprofiles)
  Type(chain), Target      :: chain_profiles_k(nchannels)
  Type(pchain)             :: p_chain_profiles_k(nchannels)
  Integer(Kind=jpim)       :: iprof, ichan, i
  Real(Kind=jprb), Pointer :: x_profiles, x_profiles_inc, x_profiles_bf, x_profiles_k
  Integer                  :: err, size
  Real(Kind=jprb)          :: inc(nprofiles)
  Logical                  :: t

  Call rttov_alloc_prof( allocstatus, nprofiles, profiles_bf, profiles(1)%nlevels, coef_scatt_ir, 1_jpim, &
     & addclouds = addclouds, addaerosl = addaerosl, init = .true. )

  Call CopyProfiles( profiles_bf, profiles, nprofiles )
     
  Call rttov_alloc_prof( allocstatus, nprofiles, profiles_inc, profiles(1)%nlevels, coef_scatt_ir, 1_jpim, &
     & addclouds = addclouds, addaerosl = addaerosl, init = .true. )
     
  Call rttov_alloc_rad( allocstatus, nchannels, radiancedata_bf, profiles(1)%nlevels, 1_jpim )
  
  Allocate( transmission_bf%tau_layers( profiles(1)%nlevels, nchannels ) )
  Allocate( transmission_bf%tau_total( nchannels ) )


  Call rttov_make_profile_inc( profiles_inc, profiles, nprofiles )
  
  Call rttov_direct(                   &
      & errorstatus,                   &
      & nprofiles,                     &
      & nchannels,                     &
      & channels,                      &
      & lprofiles,                     &
      & addinterp,                     &
      & profiles,                      &
      & coef,                          &
      & coef_scatt_ir,                 &
      & optp,                          &
      & addsun,                        &
      & addaerosl,                     &
      & addclouds,                     &
      & calcemis,                      &
      & emissivity,                    &
      & transmission,                  &
      & radiancedata                   &
      & )

  
  Do iprof = 1, nprofiles
    Call chain_profile_type( err, chain_profiles(iprof),     "PROFILES",     a0 = profiles(iprof) )
    Call chain_profile_type( err, chain_profiles_inc(iprof), "PROFILES_INC", a0 = profiles_inc(iprof) )
    Call chain_profile_type( err, chain_profiles_bf(iprof),  "PROFILES_BF",  a0 = profiles_bf(iprof) )
    p_chain_profiles(iprof)%p     => chain_profiles(iprof)
    p_chain_profiles_inc(iprof)%p => chain_profiles_inc(iprof)
    p_chain_profiles_bf(iprof)%p  => chain_profiles_bf(iprof)
  EndDo


  Do ichan = 1, nchannels
    Call chain_profile_type( err, chain_profiles_k(ichan), "PROFILES_K", a0 = profiles_k(ichan) )
    p_chain_profiles_k(ichan)%p => chain_profiles_k(ichan)
  EndDo



  Call size_chain( size, chain_profiles(1) )
  
  Do i = 1, size
    
    Do iprof = 1, nprofiles
      Call get_pointer_chain( p_chain_profiles(iprof)%p,     x_profiles )
      Call get_pointer_chain( p_chain_profiles_bf(iprof)%p,  x_profiles_bf )
      Call get_pointer_chain( p_chain_profiles_inc(iprof)%p, x_profiles_inc )
      x_profiles_bf = x_profiles + x_profiles_inc
      inc(iprof) = x_profiles_inc
    EndDo
    
    
    Call rttov_direct(                   &
        & errorstatus,                   &
        & nprofiles,                     &
        & nchannels,                     &
        & channels,                      &
        & lprofiles,                     &
        & addinterp,                     &
        & profiles_bf,                   &
        & coef,                          &
        & coef_scatt_ir,                 &
        & optp,                          &
        & addsun,                        &
        & addaerosl,                     &
        & addclouds,                     &
        & calcemis,                      &
        & emissivity,                    &
        & transmission_bf,               &
        & radiancedata_bf                &
        & )

    If( switchrad ) Then
      profiles_k(1)%q(1) = ( radiancedata_bf%bt(1) - radiancedata%bt(1) ) / profiles_inc(1)%q(1)
    Else
      profiles_k(1)%q(1) = ( radiancedata_bf%total(1) - radiancedata%total(1) ) / profiles_inc(1)%q(1)
    EndIf
    

    Do iprof = 1, nprofiles
      Call get_pointer_chain( p_chain_profiles(iprof)%p,     x_profiles )
      Call get_pointer_chain( p_chain_profiles_bf(iprof)%p,  x_profiles_bf )
      x_profiles_bf = x_profiles
    EndDo

    Do iprof = 1, nprofiles
      Call advance_chain( p_chain_profiles(iprof)%p )
      Call advance_chain( p_chain_profiles_bf(iprof)%p )
      Call advance_chain( p_chain_profiles_inc(iprof)%p )
    EndDo

    Do ichan = 1, nchannels
      Call get_pointer_chain( p_chain_profiles_k(ichan)%p, x_profiles_k )
      
      If( inc(lprofiles(ichan)) .ne. 0._jprb ) Then
        If( switchrad ) Then
          x_profiles_k = ( radiancedata_bf%bt(ichan) - radiancedata%bt(ichan) ) / inc(lprofiles(ichan))
        Else
          x_profiles_k = ( radiancedata_bf%total(ichan) - radiancedata%total(ichan) ) / inc(lprofiles(ichan))
        EndIf
      Else
        x_profiles_k = 0._jprb
      EndIf
      
      Call advance_chain( p_chain_profiles_k(ichan)%p )
    EndDo
    

  EndDo



  calcemis_bf = .false. 

  Do ichan = 1, nchannels  
    
    emissivity_bf = emissivity
    emissivity_bf(ichan) = emissivity_bf(ichan) - 0.01_jprb

    Call rttov_direct(                   &
        & errorstatus,                   &
        & nprofiles,                     &
        & nchannels,                     &
        & channels,                      &
        & lprofiles,                     &
        & addinterp,                     &
        & profiles,                      &
        & coef,                          &
        & coef_scatt_ir,                 &
        & optp,                          &
        & addsun,                        &
        & addaerosl,                     &
        & addclouds,                     &
        & calcemis_bf,                   &
        & emissivity_bf,                 &
        & transmission_bf,               &
        & radiancedata_bf                &
        & )

    If( switchrad ) Then
      emissivity_k(ichan) = ( radiancedata_bf%bt(ichan) - radiancedata%bt(ichan) ) / ( -0.01_jprb )
    Else
      emissivity_k(ichan) = ( radiancedata_bf%total(ichan) - radiancedata%total(ichan) ) / ( -0.01_jprb )
    EndIf


  EndDo



  Do ichan = 1, nchannels
    Call free_chain( chain_profiles_k(ichan) )
  EndDo
  
  
  Do iprof = 1, nprofiles
    Call free_chain( chain_profiles(iprof))
    Call free_chain( chain_profiles_inc(iprof) )
    Call free_chain( chain_profiles_bf(iprof) )
  EndDo


  DeAllocate( transmission_bf%tau_layers )
  DeAllocate( transmission_bf%tau_total )


  Call rttov_alloc_rad( allocstatus, nchannels, radiancedata_bf, profiles(1)%nlevels, 0_jpim )
  
  Call rttov_alloc_prof( allocstatus, nprofiles, profiles_inc, profiles(1)%nlevels, coef_scatt_ir, 0_jpim, &
     & addclouds = addclouds, addaerosl = addaerosl )
     
  Call rttov_alloc_prof( allocstatus, nprofiles, profiles_bf, profiles(1)%nlevels, coef_scatt_ir, 0_jpim, &
     & addclouds = addclouds, addaerosl = addaerosl )


Contains

Subroutine CopyProfiles( profiles1, profiles2, nprofiles )
Integer(Kind=jpim), Intent(in)    :: nprofiles
Type(profile_type), Intent(inout), Target :: profiles1(nprofiles)
Type(profile_type), Intent(in),    Target :: profiles2(nprofiles)
Integer(Kind=jpim)       :: iprof, i
Type(chain),     Target  :: chain1,  chain2
Type(chain),     Pointer :: pchain1, pchain2
Real(Kind=jprb), Pointer :: x1, x2
Integer                  :: err    

  Do iprof = 1, nprofiles
    profiles1(iprof)%nlevels        = profiles2(iprof)%nlevels
    profiles1(iprof)%ozone_Data     = profiles2(iprof)%ozone_Data  
    profiles1(iprof)%co2_Data       = profiles2(iprof)%co2_Data  
    profiles1(iprof)%co_Data        = profiles2(iprof)%co_Data  
    profiles1(iprof)%n2o_Data       = profiles2(iprof)%n2o_Data  
    profiles1(iprof)%ch4_Data       = profiles2(iprof)%ch4_Data  
    profiles1(iprof)%aer_Data       = profiles2(iprof)%aer_Data  
    profiles1(iprof)%cld_Data       = profiles2(iprof)%cld_Data  
    profiles1(iprof)%clw_Data       = profiles2(iprof)%clw_Data  
    profiles1(iprof)%addsolar       = profiles2(iprof)%addsolar  
    profiles1(iprof)%idg            = profiles2(iprof)%idg
    profiles1(iprof)%ish            = profiles2(iprof)%ish
    profiles1(iprof)%skin%surftype  = profiles2(iprof)%skin%surftype
    profiles1(iprof)%skin%watertype = profiles2(iprof)%skin%watertype
    profiles1(iprof)%addsolar       = profiles2(iprof)%addsolar 
    profiles1(iprof)%addrefrac      = profiles2(iprof)%addrefrac 
    profiles1(iprof)%addaerosl      = profiles2(iprof)%addaerosl 
    profiles1(iprof)%addwatrcl      = profiles2(iprof)%addwatrcl 
  EndDo

  Call chain_profile_type( err, chain1, "PROFILES1", a1 = profiles1 )
  pchain1 => chain1
  Call chain_profile_type( err, chain2, "PROFILES2", a1 = profiles2 )
  pchain2 => chain2
  
  Do While( .true. )
    Call get_pointer_chain( pchain1, x1 )
    Call get_pointer_chain( pchain2, x2 )
    If( .not. Associated( x1 ) ) Exit
    If( .not. Associated( x2 ) ) Exit
    
    x1 = x2
    
    
    Call advance_chain( pchain1 )
    Call advance_chain( pchain2 )
  EndDo
  
  Call free_chain( chain1 )
  Call free_chain( chain2 )

End Subroutine

End Subroutine rttov_k_bf
