Subroutine tstrad_k(    &
      & nprofiles,       &! in
      & nchannels,       &! in
      & channels,        &! in
      & lprofiles,       &! in
      & addinterp,       &! in
      & profiles,        &! in
      & coef,            &! in
      & coef_scatt_ir,   &! in
      & optp,            &! in
      & addsun,          &! in
      & addclouds,       &! in
      & addaerosl,       &! in
      & calcemis,        &! in
      & input_emissivity) ! in 
  !
  ! only the first nchannels/nprofiles are output
  !

  Use rttov_const, Only :   &
        & errorstatus_success ,&
        & errorstatus_fatal,   & 
        & q_mixratio_to_ppmv, &
        & o3_mixratio_to_ppmv, &
        & co2_mixratio_to_ppmv,& 
        & co_mixratio_to_ppmv, & 
        & n2o_mixratio_to_ppmv,& 
        & ch4_mixratio_to_ppmv,&
        & lgradp

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use rttov_types, Only :       & 
        & rttov_coef,           &
        & rttov_coef_scatt_ir,  &
        & rttov_optpar_ir,      &
        & profile_Type,         &
        & transmission_Type,    &
        & radiance_Type,        &
        & rttov_temp 

  Use mod_tstrad

  Use parkind1, Only : jpim     ,jprb
  Implicit None


  !subroutine arguments:
  Integer(Kind=jpim),  Intent(in)    :: nprofiles                ! Number of profiles
  Integer(Kind=jpim),  Intent(in)    :: nchannels                ! Number of radiances computed (channels used * profiles)
  Integer(Kind=jpim),  Intent(in)    :: channels(nchannels)
  Integer(Kind=jpim),  Intent(in)    :: lprofiles(nchannels)
  Logical,             Intent(in)    :: addsun
  Logical,             Intent(in)    :: addaerosl
  Logical,             Intent(in)    :: addclouds
  Logical,             Intent(in)    :: addinterp  ! switch for the interpolator
!  Type(profile_Type),  Intent(in)    :: profiles(nprofiles)
  Type(profile_Type),  Intent(inout)    :: profiles(nprofiles)

  Type(rttov_coef),    Intent(in)    :: coef
  type(rttov_coef_scatt_ir) ,    Intent(in)    :: coef_scatt_ir
  type(rttov_optpar_ir), Intent(in)    :: optp
  Logical,             Intent(in)    :: calcemis(nchannels)
  Real(Kind=jprb),     Intent(in)    :: input_emissivity(nchannels)

!INTF

#include "rttov_errorreport.interface"
#ifdef _RTTOV_TSTRAD_PARALLEL
#include "rttov_parallel_k.interface"
#define rttov_k rttov_parallel_k
#else
#include "rttov_k.interface"
#endif
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_temp.interface"

  ! local
  Integer(Kind=jpim), Parameter :: jpnav  =  31      ! no. of profile variables
  Integer(Kind=jpim), Parameter :: jpnsav =  6       ! no. of surface air variables
  Integer(Kind=jpim), Parameter :: jpnssv =  6       ! no. of skin variables
  Integer(Kind=jpim), Parameter :: jpncv  =  2       ! no. of cloud variables
  Integer(Kind=jpim), Parameter :: sscvar = jpnsav+jpnssv+jpncv ! no of surface,skin,cloud vars

  Integer(Kind=jpim)            :: errorstatus
  Character (len=80) :: errMessage
  Character (len=10) :: NameOfRoutine = 'tstrad_k  '


  type(rttov_temp)   :: temp, temp_tl, temp_ad, temp_k
  Integer(Kind=jpim) :: temp_status

  ! forward model outputs
  logical :: switchrad
  Type(transmission_Type)   :: transmission
  Type(radiance_Type)   :: radiancedata
  Real(Kind=jprb)   :: emissivity(nchannels)


  ! AD variables for rttov_k calls
  Type(profile_Type)  :: profiles_k(nchannels)
  Type(transmission_Type)   :: transmission_k
  Real(Kind=jprb)  :: emissivity_k(nchannels)

  Integer(Kind=jpim) :: nlev
  Integer(Kind=jpim) :: ixkav(profiles(1)%nlevels,jpnav,nchannels)
  Integer(Kind=jpim) :: ixksav(sscvar,nchannels)

  ! Adjoint results
  Integer(Kind=jpim) :: ixkdav(profiles(1)%nlevels,jpnav,nchannels)
  Real(Kind=jprb)    :: xktav (profiles(1)%nlevels,jpnav,nchannels)
  Integer(Kind=jpim) :: ixkdsav(sscvar,nchannels)
  Real(Kind=jprb)    :: xktsav (sscvar,nchannels)
  Integer(Kind=jpim) :: ixdem(nchannels)
  Real(Kind=jprb)    :: xkaem(nchannels)

  ! coefficients for printing
  Real(Kind=jprb) :: facpav(profiles(1)%nlevels,jpnav)
  Real(Kind=jprb) :: facovu(profiles(1)%nlevels)
  Real(Kind=jprb) :: facovd(profiles(1)%nlevels)
  Real(Kind=jprb) :: facem = 1._JPRB

  Real(Kind=jprb) :: facsav(sscvar) =&
         & (/10000._JPRB,0.1_JPRB,10000._JPRB,10000._JPRB,10000._JPRB,0.1_JPRB,       &! 2m
         & 10000._JPRB,100.0_JPRB,100.0_JPRB,100.0_JPRB,100.0_JPRB,100.0_JPRB,  &! Skin
         & 10000._JPRB,100._JPRB/)                           ! cloud 

  Real(Kind=jprb) :: facdiff = 1.e+10_JPRB
  ! Real :: facdiff = 1.

  Integer(Kind=jpim) :: ioout = 2
  Integer(Kind=jpim) :: ich, jch
  Integer(Kind=jpim) :: j, i, ii, jp, joff
  Integer(Kind=jpim) :: nchan_out, nprint

  Character (len=30) :: title(31) = &
         & (/' lev       temperature        ', &
           & ' lev       water vapour       ',   &
           & ' lev       ozone              ',   &
           & ' lev       co2                ',   &
           & ' lev       n2o                ',   &
           & ' lev       co                 ',   &
           & ' lev       ch4                ',   &
           & ' lev       aer1               ',   &
           & ' lev       aer2               ',   &
           & ' lev       aer3               ',   &
           & ' lev       aer4               ',   &
           & ' lev       aer5               ',   &
           & ' lev       aer6               ',   &
           & ' lev       aer7               ',   &
           & ' lev       aer8               ',   &
           & ' lev       aer9               ',   &
           & ' lev       aer10              ',   &
           & ' lev       aer11              ',   &
           & ' lev       wcl1               ',   &
           & ' lev       wcl2               ',   &
           & ' lev       wcl3               ',   &
           & ' lev       wcl4               ',   &
           & ' lev       wcl5               ',   &
           & ' lev       icl                ',   &
           & ' lev       cfr1               ',   &
           & ' lev       cfr2               ',   &
           & ' lev       cfr3               ',   &
           & ' lev       cfr4               ',   &
           & ' lev       cfr5               ',   &
           & ' lev       cfri               ',      &
           & ' lev       liquid water       '/) 

  Integer(Kind=jpim) :: alloc_status(60)
  Integer(Kind=jpim) :: rttov_errorstatus(nprofiles)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TSTRAD_K',0_jpim,ZHOOK_HANDLE)
  errorstatus     = 0
  alloc_status(:) = 0

  nchan_out = nchannels/nprofiles
  nlev = profiles(1) % nlevels

  ! coefficients for atmospheric variables
  facpav(:,1) = 10000._JPRB
  facpav(:,2) = 0.1_JPRB
  facpav(:,3) = 0.001_JPRB
  facpav(:,4) = 0.001_JPRB
  facpav(:,5) = 0.001_JPRB
  facpav(:,6) = 0.001_JPRB
  facpav(:,7) = 0.001_JPRB
  facpav(:,8) = 1000_JPRB
  facpav(:,9) = 1000_JPRB
  facpav(:,10) = 1000_JPRB
  facpav(:,11) = 1000_JPRB
  facpav(:,12) = 1000_JPRB
  facpav(:,13) = 1000_JPRB
  facpav(:,14) = 1000_JPRB
  facpav(:,15) = 1000_JPRB
  facpav(:,16) = 1000_JPRB
  facpav(:,17) = 1000_JPRB
  facpav(:,18) = 1000_JPRB
  facpav(:,19) = 100_JPRB
  facpav(:,20) = 100_JPRB
  facpav(:,21) = 100_JPRB
  facpav(:,22) = 100_JPRB
  facpav(:,23) = 100_JPRB
  facpav(:,24) = 100_JPRB
  facpav(:,25) = 100_JPRB
  facpav(:,26) = 100_JPRB
  facpav(:,27) = 100_JPRB
  facpav(:,28) = 100_JPRB
  facpav(:,29) = 100_JPRB
  facpav(:,30) = 100_JPRB
  facpav(:,31) = 0.1_JPRB
  facovu(:)   = 10000._JPRB
  facovd(:)   = 100000._JPRB

  ! coefficients compatibility with RTTOV7
  facpav(:,2) = facpav(:,2) *  q_mixratio_to_ppmv
  facpav(:,3) = facpav(:,3) * o3_mixratio_to_ppmv
  facpav(:,4) = facpav(:,4) * co2_mixratio_to_ppmv
  facpav(:,5) = facpav(:,5) * n2o_mixratio_to_ppmv
  facpav(:,6) = facpav(:,6) * co_mixratio_to_ppmv
  facpav(:,7) = facpav(:,7) * ch4_mixratio_to_ppmv
  facsav(2)   = facsav(2)   * q_mixratio_to_ppmv

! allocate transmittance arrays with number of channels
   allocate( transmission % tau_layers (profiles(1) % nlevels,nchannels ) ,stat= alloc_status(1))
   allocate( transmission % tau_total (nchannels ) ,stat= alloc_status(2))
   allocate( transmission_k % tau_layers (profiles(1) % nlevels,nchannels ) ,stat= alloc_status(3))
   allocate( transmission_k % tau_total (nchannels ) ,stat= alloc_status(4))
   If( Any(alloc_status /= 0) ) Then
       errorstatus = errorstatus_fatal
       Write( errMessage, '( "allocation of transmission")' )
       Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
       IF (LHOOK) CALL DR_HOOK('TSTRAD_K',1_jpim,ZHOOK_HANDLE)
       Stop
    End If
   transmission % tau_layers = 0._jprb
   transmission % tau_total = 0._jprb
   transmission_k % tau_layers = 0._jprb
   transmission_k % tau_total = 0._jprb

  ! allocate and initialise the reference tl increments
  Call  rttov_alloc_prof( alloc_status(1), nchannels, profiles_k, profiles(1) % nlevels, coef_scatt_ir, 1_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds, init=.true.)
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata, profiles(1) % nlevels, 1_jpim )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If
  !
  !...do K.....................
  ! use stored input emmissivity
  emissivity(:) = input_emissivity(:)


#ifdef _RTTOV_TSTRAD_TEMP
  call rttov_alloc_temp(                         &
      & temp_status,                             &
      & nprofiles,                               &
      & nchannels,                               &
      & channels,                                &
      & addinterp,                               &
      & profiles(1)%nlevels,                     &
      & coef,                                    &
      & coef_scatt_ir,                           &
      & optp,                                    &
      & addsun .or. Any( profiles(:)%addsolar ), &
      & addaerosl,                               &
      & addclouds,                               &
      & 1_jpim,                                  &
      & temp = temp, temp_k = temp_k )     
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Stop       
  EndIf     
#endif


  Do i = 1, nchannels
     profiles_k(i) % ozone_Data = .False.  ! no meaning
     profiles_k(i) % co2_Data   = .False.  ! no meaning
     profiles_k(i) % n2o_Data   = .False.  ! no meaning
     profiles_k(i) % co_Data    = .False.  ! no meaning
     profiles_k(i) % ch4_Data   = .False.  ! no meaning
     profiles_k(i) % clw_Data   = .False.  ! no meaning
     profiles_k(i) % zenangle   = -1._jprb ! no meaning
     profiles_k(i) % azangle    = -1._jprb ! no meaning
     profiles_k(i) % sunzenangle= -1._jprb ! no meaning
     profiles_k(i) % sunazangle = -1._jprb ! no meaning
     profiles_k(i) % latitude   = -1._jprb ! no meaning
     profiles_k(i) % elevation  = -1._jprb ! no meaning

     ! increments for atmospheric variables
     profiles_k(i) % p(:)   = 0._JPRB ! pressure levels
     profiles_k(i) % t(:)   = 0._JPRB ! temperarure
     profiles_k(i) % o3(:)  = 0._JPRB ! O3 ppmv
     profiles_k(i) % co2(:) = 0._JPRB ! CO2 ppmv
     profiles_k(i) % n2o(:) = 0._JPRB ! N2O ppmv
     profiles_k(i) % co(:)  = 0._JPRB ! CO ppmv
     profiles_k(i) % ch4(:) = 0._JPRB ! CH4 ppmv
     if(addaerosl)then
          profiles_k(i) % aerosols(:,:) = 0._JPRB
        endif
        if(addclouds)then
          profiles_k(i) % cloud(:,:)=0._JPRB
          profiles_k(i) % cfrac(:,:)=0._JPRB
        endif
     profiles_k(i) % clw(:) = 0._JPRB ! clw
     profiles_k(i) % q(:)   = 0._JPRB ! WV

     ! increments for air surface variables
     profiles_k(i) % s2m % t = 0._JPRB !  temperarure
     profiles_k(i) % s2m % q = 0._JPRB !  WV
     profiles_k(i) % s2m % o = 0._JPRB !  O3
     profiles_k(i) % s2m % p = 0._JPRB !  pressure
     profiles_k(i) % s2m % u = 0._JPRB !  wind components
     profiles_k(i) % s2m % v = 0._JPRB !  wind components
     profiles_k(i) % s2m % wfetc= 0._JPRB!

     ! increments for skin variables
     profiles_k(i) % skin % surftype = -1  ! no meaning
     profiles_k(i) % skin % t        = 0._JPRB  ! on temperarure
     profiles_k(i) % skin % fastem   = 0._JPRB

     ! increments for cloud variables
     profiles_k(i) % ctp       = 0._JPRB  ! pressure
     profiles_k(i) % cfraction = 0._JPRB  ! cloud fraction
  End Do

  emissivity_k(:) = 0._JPRB

  switchrad= .true.
  call rttov_k( &
        & rttov_errorstatus,  &! out
        & nprofiles,          &! in
        & nchannels,          &! in
        & channels,           &! in
        & lprofiles,          &! in
        & addinterp,          &! in
        & profiles,           &! in
        & profiles_k,         &! inout  adjoint
        & coef,               &! in
        & coef_scatt_ir,      &
        & optp,               &
        & addsun,             &! in
        & addaerosl,          &
        & addclouds,          &
        & switchrad,          &! in
        & calcemis,           &! in
        & emissivity,         &! inout  direct model
        & emissivity_k,       &! inout  adjoint
        & transmission,       &! inout  adjoint
        & transmission_k,     &! inout  adjoint
        & radiancedata        &! inout  direct model 
#ifdef _RTTOV_TSTRAD_TEMP
        & , temp = temp, temp_k = temp_k &
#endif
        & )        


        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
          Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_k")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
       If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_k")' )           
        End If




  Do  ich =1, nchannels
     Loop1: Do  j =1,jpnav    ! yes clw too!
        !Write(0,*) 'adjoint de la variable atmosphere ',j
        ii = 1
              xktav(ii,j,ich) = profiles_k(ich) % t(ii)
        Do  ii=1,nlev

           if((addaerosl).and.(.NOT.addclouds))then
             if(j>=19.and.j<=30)then
               ixkdav(:,j,ich)=0_jpim
               cycle loop1
             endif
           else if ((.NOT.addaerosl).and.(addclouds))then
             if(j>=8.and.j<=18)then
               ixkdav(:,j,ich)=0_jpim
               cycle loop1
             endif
           else if ((.NOT.addaerosl).and.(.NOT.addclouds))then
             if(j>=8.and.j<=30)then
               ixkdav(:,j,ich)=0_jpim
               cycle loop1
             endif
           endif

          Select Case (j)
           Case (1_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % t(ii)
           Case (2_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % q(ii)
           Case (3_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % o3(ii)
           Case (4_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % co2(ii)
           Case (5_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % n2o(ii)
           Case (6_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % co(ii)
           Case (7_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % ch4(ii)
           Case (8_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(1,ii)
           Case (9_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(2,ii)
           Case (10_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(3,ii)
           Case (11_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(4,ii)
           Case (12_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(5,ii)
           Case (13_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(6,ii)
           Case (14_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(7,ii)
           Case (15_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(8,ii)
           Case (16_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(9,ii)
           Case (17_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(10,ii)
           Case (18_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % aerosols(11,ii)
           Case (19_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cloud(1,ii)
           Case (20_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cloud(2,ii)
           Case (21_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cloud(3,ii)
           Case (22_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cloud(4,ii)
           Case (23_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cloud(5,ii)
           Case (24_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cloud(6,ii)
           Case (25_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cfrac(1,ii)
           Case (26_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cfrac(2,ii)
           Case (27_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cfrac(3,ii)
           Case (28_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cfrac(4,ii)
           Case (29_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cfrac(5,ii)
           Case (30_jpim)
              xktav(ii,j,ich) = profiles_k(ich) % cfrac(6,ii)
           Case (31_jpim) 
              xktav(ii,j,ich) = profiles_k(ich) % clw(ii)
           End Select

          if(lprofiles(ich) == 1) then
               !Write(0,*) 'level xkb, xkt ',ii,j,ich,xkbav(ii,j,ich),xktav(ii,j,ich)

              ixkav(ii,j,ich) = nint(xktav(ii,j,ich) * facpav(ii,j))
              ixkdav(ii,j,ich) = nint((xkbav(ii,j,ich)-xktav(ii,j,ich)) *&
                      & facpav(ii,j) * facdiff) 

           endif
        End Do
     End Do loop1
 
 
     !.......now do surface, skin and cloud variables
     Do  j =1,sscvar

        Select Case (j)
        Case (1_jpim)
           ! t 2m
           xktsav(j,ich) = profiles_k(ich) % s2m % t
        Case (2_jpim)
           ! wv 2m
           xktsav(j,ich) = profiles_k(ich) % s2m % q
        Case (3_jpim)
           ! surface pressure
           xktsav(j,ich) = profiles_k(ich) % s2m % p
        Case (4_jpim)
           ! wind speed u component
           xktsav(j,ich) = profiles_k(ich) % s2m % u
        Case (5_jpim)
           ! wind speed v component
           xktsav(j,ich) = profiles_k(ich) % s2m % v
        Case (6_jpim)
           ! wind fetch component
           xktsav(j,ich) = profiles_k(ich) % s2m % wfetc
        Case (7_jpim)
           ! skin temp
           xktsav(j,ich) = profiles_k(ich) % skin % t
        Case (8_jpim)
           ! fastem land coef 1
           xktsav(j,ich) = profiles_k(ich) % skin % fastem(1)
        Case (9_jpim)
           ! fastem land coef 2
           xktsav(j,ich) = profiles_k(ich) % skin % fastem(2)
        Case (10_jpim)
           ! fastem land coef 3
           xktsav(j,ich) = profiles_k(ich) % skin % fastem(3)
        Case (11_jpim)
           ! fastem land coef 4
           xktsav(j,ich) = profiles_k(ich) % skin % fastem(4)
        Case (12_jpim)
           ! fastem land coef 5
           xktsav(j,ich) = profiles_k(ich) % skin % fastem(5)
        Case (13_jpim)
           ! cloud top pressure
           xktsav(j,ich) = profiles_k(ich) % ctp
        Case (14_jpim)
           ! cloud fraction
           xktsav(j,ich) = profiles_k(ich) % cfraction
        End Select
     End Do

     if(lprofiles(ich) == 1) then
       ixksav(:,ich)  = nint(xktsav(:,ich) * facsav(:))

       ixkdsav(:,ich) = nint((xkbsav(:,ich)-xktsav(:,ich)) *&
         & facsav(:) * facdiff) 
     endif

     xkaem(ich) = 0.0_JPRB
     xkaem(ich) = xkaem(ich) + emissivity_k(ich)
     ixdem(ich) = nint(( xkaem(ich) - xkbem(ich)) * facem *1000._JPRB)

     If( coef % fastem_ver >= 1 .and. calcemis(ich) ) Then
     !if(input_emissivity(ich) < 0.0_JPRB ) then
        ixdem(ich) = 0._JPRB
     endif

  end do

  !          ... and print it.
  Write (ioout,*)' '
  Write (ioout,*)'TL - K difference*10**10.'
  Write (ioout,*)' '
  If (addclouds .or. addaerosl)Then
    nprint = jpnav 
  Else
    nprint = 4
  Endif

!! optional extra tables - explicit Jacobians from K-code
!  Do j = 1 , nprint                   ! lwp on
!     Write (ioout,'(''optional: explicit Jacobians from K-code'',a30)')title(j)
!     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
!     Do i = 1 , nlev
!        Write (ioout,333)i,(ixkav(i,j,jch),jch=1,nchan_out)
!     Enddo
!     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
!     Write (ioout,*)' '
!  Enddo

! Jacobian differences (TL - K)
  Do j = 1 , nprint                   ! lwp on
     Write (ioout,'(a30)')title(j)
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Do i = 1 , nlev
        Write (ioout,333)i,(ixkdav(i,j,jch),jch=1,nchan_out)
     Enddo
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Write (ioout,*)' '
  Enddo

  Write (ioout,*)' surface variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = 1 , jpnsav
     Write (ioout,333)i,(ixkdsav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' skin variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = jpnsav+1 , jpnsav+jpnssv
     Write (ioout,333)i-jpnsav,(ixkdsav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' cloud variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = jpnsav+jpnssv+1 , sscvar
     Write (ioout,333)i-jpnsav-jpnssv,(ixkdsav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' surface emissivity '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do jp = 1, nprofiles
     joff = (jp-1) * nchan_out
     Write (ioout,333)jp,(ixdem(jch+joff),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  write(ioout,*)' calculated br. temps in rttovk'
  write(ioout,222) (radiancedata % bt (1:nchan_out))
  write(ioout,*)' '
  write(ioout,*)' calculated radiances in rttovk'
  write(ioout,222) (radiancedata % total (1:nchan_out))
  write(ioout,*)' '
  write (ioout,*)' '



#ifdef _RTTOV_TSTRAD_TEMP
  call rttov_alloc_temp(                         &
      & temp_status,                             &
      & nprofiles,                               &
      & nchannels,                               &
      & channels,                                &
      & addinterp,                               &
      & profiles(1)%nlevels,                     &
      & coef,                                    &
      & coef_scatt_ir,                           &
      & optp,                                    &
      & addsun .or. Any( profiles(:)%addsolar ), &
      & addaerosl,                               &
      & addclouds,                               &
      & 0_jpim,                                  &
      & temp = temp, temp_k = temp_k )     
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Stop       
  EndIf     
#endif


  Call  rttov_alloc_prof( alloc_status(1), nchannels, profiles_k, profiles(1) % nlevels, coef_scatt_ir, 0_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem deallocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  ! deallocate transmittances
  Deallocate( transmission % tau_total   ,stat= alloc_status(9))
  Deallocate( transmission % tau_layers  ,stat= alloc_status(10))
  Deallocate( transmission_k % tau_total   ,stat= alloc_status(11))
  Deallocate( transmission_k % tau_layers  ,stat= alloc_status(12))

  ! deallocate radiance results arrays with number of channels
  Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata, profiles(1) % nlevels, 0_jpim )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem deallocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_K',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  Print *, ' K matrix test finished'
IF (LHOOK) CALL DR_HOOK('TSTRAD_K',1_jpim,ZHOOK_HANDLE)
222 Format(1x,10f8.2)
333 Format(1x,i3,20i5)
444 Format(1x,10e8.2)
3333 Format(4x,20i5)
4444 Format(1x,10f8.3)

End Subroutine tstrad_k
