Subroutine tstrad_tl(   &
      & nprofiles,       &! in
      & nchannels,       &! in
      & channels,        &! in
      & lprofiles,       &! in
      & addinterp,       &! in
      & profiles,        &! in
      & coef,            &! in
      & coef_scatt_ir,   &! in
      & optp,            &! in
      & addsun,          &! in
      & addclouds,       &! in
      & addaerosl,       &! in
      & calcemis,        &! in
      & input_emissivity) ! in 
  !
  ! only the first nchannels/nprofiles are output
  !
  Use rttov_const, Only :   &
        & errorstatus_success, &
        & errorstatus_fatal,   &
        & sensor_id_mw,        & 
        & sensor_id_po,        & 
        & q_mixratio_to_ppmv,  &
        & o3_mixratio_to_ppmv, &
        & co2_mixratio_to_ppmv,& 
        & co_mixratio_to_ppmv, & 
        & n2o_mixratio_to_ppmv,& 
        & ch4_mixratio_to_ppmv,&
        & lgradp

USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

  Use rttov_types, Only :        &
        & rttov_coef          ,  &
        & rttov_coef_scatt_ir ,  &
        & rttov_optpar_ir     ,  &
        & profile_Type        ,  &
        & transmission_Type   ,  &
        & radiance_Type       ,  &
        & rttov_temp

  Use mod_tstrad

  Use parkind1, Only : jpim     ,jprb
  Implicit None


  !subroutine arguments:
  Integer(Kind=jpim),  Intent(in)    :: nprofiles                ! Number of profiles
  Integer(Kind=jpim),  Intent(in)    :: nchannels                ! Number of radiances computed (channels used * profiles)
  Integer(Kind=jpim),  Intent(in)    :: channels(nchannels)      ! Channel indices
  Integer(Kind=jpim),  Intent(in)    :: lprofiles(nchannels)
  Logical,             Intent(in)    :: addsun
  Logical,             Intent(in)    :: addclouds
  Logical,             Intent(in)    :: addaerosl
  Logical,             Intent(in)    :: addinterp  ! switch for the interpolator
  Type(profile_Type),  Intent(inout) :: profiles(nprofiles)
  Type(rttov_coef),    Intent(in)    :: coef
  type(rttov_coef_scatt_ir) ,    Intent(in) :: coef_scatt_ir
  type(rttov_optpar_ir), Intent(in)    :: optp
  Logical,             Intent(in)    :: calcemis(nchannels)
  Real(Kind=jprb),     Intent(in)    :: input_emissivity(nchannels)

!INTF

#include "rttov_errorreport.interface"
#ifdef _RTTOV_TSTRAD_PARALLEL
#include "rttov_parallel_direct.interface"
#include "rttov_parallel_tl.interface"
#define rttov_direct rttov_parallel_direct
#define rttov_tl rttov_parallel_tl
#else
#include "rttov_direct.interface"
#include "rttov_tl.interface"
#endif
#include "rttov_alloc_prof.interface"
#include "rttov_alloc_rad.interface"
#include "rttov_alloc_temp.interface"


  ! local
  Integer(Kind=jpim), Parameter :: jpnav  =  31    ! no. of profile variables
  Integer(Kind=jpim), Parameter :: jpnsav =  6       ! no. of surface air variables
  Integer(Kind=jpim), Parameter :: jpnssv =  6       ! no. of skin variables
  Integer(Kind=jpim), Parameter :: jpncv  =  2       ! no. of cloud variables
  Integer(Kind=jpim), Parameter :: sscvar = jpnsav+jpnssv+jpncv ! no of surface,skin,cloud vars

  Character (len=80) :: errMessage
  Character (len=10) :: NameOfRoutine = 'tstrad_tl '

  type(rttov_temp)   :: temp, temp_tl, temp_ad, temp_k
  Integer(Kind=jpim) :: temp_status

  ! forward model outputs
  Type(transmission_Type)   :: transmission
  Type(radiance_Type)   :: radiancedata
  Type(radiance_Type)   :: radiance_fwd
  Real(Kind=jprb)   :: emissivity_fwd(nchannels)
  Real(Kind=jprb)   :: emissivity(nchannels)


  ! tl increments
  Type(profile_Type)  :: prof_inc(nprofiles)
  Type(profile_Type)  :: null_inc(nprofiles) ! all increments set to 0
  Real(Kind=jprb) :: emissivity_inc(nchannels)
  Real(Kind=jprb) :: null_emissivity_inc(nchannels)

  ! tl variables for rttov_tl calls
  Type(profile_Type)  :: profiles_tl(nprofiles)
  Type(transmission_Type)   :: transmission_tl
  Type(radiance_Type) :: radiancedata_tl
  Real(Kind=jprb)  :: emissivity_tl(nchannels)

  ! Brute force
  Type(profile_Type)    :: profiles_bf(nprofiles)
  Real(Kind=jprb)     :: emissivity_bf(nchannels)
  Logical  :: calcemis_bf(nchannels)

  Integer(Kind=jpim) :: nlev
  Integer(Kind=jpim) :: ixkav(profiles(1)%nlevels,jpnav,nchannels)
  Integer(Kind=jpim) :: ixkov(profiles(1)%nlevels,nchannels)
  Real(Kind=jprb)    :: pktav(profiles(1)%nlevels,jpnav,nchannels)

  Integer(Kind=jpim) :: ixkem(nchannels)

  Integer(Kind=jpim) :: ixksav(sscvar,nchannels)
  Real(Kind=jprb)    :: pktsav(sscvar,nchannels)

  ! Brute force results
  Integer(Kind=jpim) :: ixkdav(profiles(1)%nlevels,jpnav,nchannels)
  Real(Kind=jprb)    :: xktav (profiles(1)%nlevels,jpnav,nchannels)
  Integer(Kind=jpim) :: ixkdsav(sscvar,nchannels)
  Real(Kind=jprb)    :: xktsav (sscvar,nchannels)
  Integer(Kind=jpim) :: ixkdem(nchannels)
  Real(Kind=jprb)    :: xktem (nchannels)

  ! coefficients for printing
  Real(Kind=jprb) :: facpav(profiles(1)%nlevels,jpnav)
  Real(Kind=jprb) :: facovu(profiles(1)%nlevels)
  Real(Kind=jprb) :: facovd(profiles(1)%nlevels)
  Real(Kind=jprb) :: facem = 1._JPRB

  Real(Kind=jprb) :: facsav(sscvar) =&
         & (/10000._JPRB,0.1_JPRB,10000._JPRB,10000._JPRB,10000._JPRB,0.1_JPRB,       &! 2m
         & 10000._JPRB,100.0_JPRB,100.0_JPRB,100.0_JPRB,100.0_JPRB,100.0_JPRB,  &! Skin
         & 10000._JPRB,100._JPRB/)                           ! cloud 

  Real(Kind=jprb) :: inc_val   ! increment value
  Real(Kind=jprb) :: inc_val_p ! increment value for printing (ppmv -> kg/kg)
  Real(Kind=jprb) :: diffr
  Real(Kind=jprb) :: sumr
  Real(Kind=jprb) :: sumrr
  Real(Kind=jprb) :: fac
  Integer(Kind=jpim) :: ioout = 2
  Integer(Kind=jpim) :: jch
  Integer(Kind=jpim) :: j, i, ii, jp, joff
  Integer(Kind=jpim) :: prof
  Integer(Kind=jpim) :: nchan_out, nprint

  Character (len=30) :: title(31) = &
         & (/' lev       temperature        ', &
           & ' lev       water vapour       ',   &
           & ' lev       ozone              ',   &
           & ' lev       co2                ',   &
           & ' lev       n2o                ',   &
           & ' lev       co                 ',   &
           & ' lev       ch4                ',   &
           & ' lev       aer1               ',   &
           & ' lev       aer2               ',   &
           & ' lev       aer3               ',   &
           & ' lev       aer4               ',   &
           & ' lev       aer5               ',   &
           & ' lev       aer6               ',   &
           & ' lev       aer7               ',   &
           & ' lev       aer8               ',   &
           & ' lev       aer9               ',   &
           & ' lev       aer10              ',   &
           & ' lev       aer11              ',   &
           & ' lev       wcl1               ',   &
           & ' lev       wcl2               ',   &
           & ' lev       wcl3               ',   &
           & ' lev       wcl4               ',   &
           & ' lev       wcl5               ',   &
           & ' lev       icl                ',   &
           & ' lev       cfr1               ',   &
           & ' lev       cfr2               ',   &
           & ' lev       cfr3               ',   &
           & ' lev       cfr4               ',   &
           & ' lev       cfr5               ',   &
           & ' lev       cfri               ',   &
           & ' lev       liquid water       '/) 

  Integer(Kind=jpim) :: errorstatus
  Integer(Kind=jpim) :: rttov_errorstatus(nprofiles)
  Integer(Kind=jpim) :: alloc_status(60)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

  !- End of header --------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',0_jpim,ZHOOK_HANDLE)
  errorstatus     = 0
  alloc_status(:) = 0
  rttov_errorstatus(:)    = 0

  nchan_out = nchannels/nprofiles
  nlev = profiles(1)%nlevels
  ! coefficients for atmospheric variables
  facpav(:,1) = 10000._JPRB
  facpav(:,2) = 0.1_JPRB
  facpav(:,3) = 0.001_JPRB
  facpav(:,4) = 0.1_JPRB
  facpav(:,5) = 0.1_JPRB
  facpav(:,6) = 0.1_JPRB
  facpav(:,7) = 0.1_JPRB
  facpav(:,8) = 1000_JPRB
  facpav(:,9) = 1000_JPRB
  facpav(:,10) = 1000_JPRB
  facpav(:,11) = 1000_JPRB
  facpav(:,12) = 1000_JPRB
  facpav(:,13) = 1000_JPRB
  facpav(:,14) = 1000_JPRB
  facpav(:,15) = 1000_JPRB
  facpav(:,16) = 1000_JPRB
  facpav(:,17) = 1000_JPRB
  facpav(:,18) = 1000_JPRB
  facpav(:,19) = 100_JPRB
  facpav(:,20) = 100_JPRB
  facpav(:,21) = 100_JPRB
  facpav(:,22) = 100_JPRB
  facpav(:,23) = 100_JPRB
  facpav(:,24) = 100_JPRB
  facpav(:,25) = 100_JPRB
  facpav(:,26) = 100_JPRB
  facpav(:,27) = 100_JPRB
  facpav(:,28) = 100_JPRB
  facpav(:,29) = 100_JPRB
  facpav(:,30) = 100_JPRB
  facpav(:,31) = 0.1_JPRB
  
  facovu(:)   = 10000._JPRB
  facovd(:)   = 100000._JPRB
  pktav(:,:,:)=0._JPRB

 ! allocate transmittance arrays with number of channels
   allocate( transmission % tau_layers (profiles(1)%nlevels,nchannels ) ,stat= alloc_status(1))
   allocate( transmission % tau_total (nchannels ) ,stat= alloc_status(2))
   allocate( transmission_tl % tau_layers (profiles(1)%nlevels,nchannels ) ,stat= alloc_status(3))
   allocate( transmission_tl % tau_total (nchannels ) ,stat= alloc_status(4))
   If( Any(alloc_status /= 0) ) Then
       errorstatus = errorstatus_fatal
       Write( errMessage, '( "allocation of transmission")' )
       Call Rttov_ErrorReport (errorstatus_fatal, errMessage, NameOfRoutine)
       IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',1_jpim,ZHOOK_HANDLE)
       Stop
    End If
   transmission % tau_layers = 0._jprb
   transmission % tau_total = 0._jprb
   transmission_tl % tau_layers = 0._jprb
   transmission_tl % tau_total = 0._jprb

  ! allocate and initialise the reference tl increments
  Call rttov_alloc_prof( alloc_status(1), nprofiles, prof_inc, profiles(1) % nlevels, coef_scatt_ir, 1_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds, init = .true. )
  Call rttov_alloc_prof( alloc_status(2), nprofiles, null_inc, profiles(1) % nlevels, coef_scatt_ir, 1_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds, init = .true. )
  Call rttov_alloc_prof( alloc_status(3), nprofiles, profiles_tl, profiles(1) % nlevels, coef_scatt_ir, 1_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds , init = .true.)
  Call rttov_alloc_prof( alloc_status(4), nprofiles, profiles_bf, profiles(1) % nlevels, coef_scatt_ir, 1_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds, init = .true. )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  ! allocate radiance results arrays with number of channels
  Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata, profiles(1)%nlevels, 1_jpim )
  Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata_tl, profiles(1)%nlevels, 1_jpim )
  Call rttov_alloc_rad( alloc_status(1), nchannels, radiance_fwd, profiles(1)%nlevels, 1_jpim )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem allocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  Do j = 1, nprofiles
     prof_inc(j) % ozone_Data = .False.  ! no meaning
     prof_inc(j) % co2_Data   = .False.  ! no meaning
     prof_inc(j) % co_Data    = .False.  ! no meaning
     prof_inc(j) % n2o_Data   = .False.  ! no meaning
     prof_inc(j) % ch4_Data   = .False.  ! no meaning
     prof_inc(j) % aer_Data   = .False.  ! no meaning
     prof_inc(j) % cld_Data   = .False.  ! no meaning
     prof_inc(j) % clw_Data   = .False.  ! no meaning 
     prof_inc(j) % addsolar   = .False.  ! no meaning
     prof_inc(j) % addrefrac  = .False.  ! no meaning
     prof_inc(j) % zenangle   = -1  ! no meaning
     prof_inc(j) % azangle    = -1
     prof_inc(j) % sunzenangle= -1
     prof_inc(j) % sunazangle = -1
     prof_inc(j) % latitude   = -1
     prof_inc(j) % elevation  = -1
     prof_inc(j) % Be  = -1
     prof_inc(j) % cosbk  = -1

     ! increments for atmospheric variables
     If( lgradp )Then
       prof_inc(j) % p(:)   = 0.01_JPRB*profiles(j) %p        ! tl on pressure levels
     Else
       prof_inc(j) % p(:)   = 0._JPRB
     EndIf
     prof_inc(j) % t(:)   = -1._JPRB                        ! 1k on temperarure
     prof_inc(j) % o3(:)  = -0.01_JPRB                      !  0.01_JPRB ppmv
     prof_inc(j) % co2(:) = -0.01_JPRB*profiles(j) % co2(:) ! - 1% on wv 
     prof_inc(j) % co(:)  = -0.01_JPRB*profiles(j) % co(:)  ! - 1% on wv 
     prof_inc(j) % n2o(:) = -0.01_JPRB*profiles(j) % n2o(:) ! - 1% on wv
     prof_inc(j) % ch4(:) = -0.01_JPRB*profiles(j) % ch4(:) ! - 1% on wv
     if(addaerosl)then
       prof_inc(j) % aerosols(:,:)=-0.01_JPRB*profiles(j) % aerosols(:,:)
     endif
     if(addclouds)then
       prof_inc(j) % cloud(:,:)=-0.01_JPRB*profiles(j) % cloud(:,:)
       prof_inc(j) % cfrac(:,:)=-0.01_JPRB*profiles(j) % cfrac(:,:)
     endif
     prof_inc(j) % clw(:) = 0.001_JPRB                      ! 1g/kg on clw
     prof_inc(j) % q(:) = -0.1_JPRB * profiles(j) % q(:)    ! - 10% on wv

!     prof_inc(j) % p(:)   = 0._JPRB                         ! no tl on pressure levels
!     prof_inc(j) % t(:)   = -0.02*profiles(j) % t(:)                        ! 1k on temperarure
!     prof_inc(j) % o3(:)  = -0.02*profiles(j) % o3(:)                      !  0.01_JPRB ppmv
!     prof_inc(j) % co2(:) = -0.02_JPRB*profiles(j) % co2(:) ! - 1% on wv
!     prof_inc(j) % co(:)  = -0.02_JPRB*profiles(j) % co(:)  ! - 1% on wv
!     prof_inc(j) % n2o(:) = -0.02_JPRB*profiles(j) % n2o(:) ! - 1% on wv
!     prof_inc(j) % ch4(:) = -0.02_JPRB*profiles(j) % ch4(:) ! - 1% on wv
!     prof_inc(j) % aerosols(:,:)=-0.02_JPRB*profiles(j) % aerosols(:,:)
!     prof_inc(j) % clw(:) = 0.001_JPRB                      ! 1g/kg on clw
!     prof_inc(j) % q(:) = -0.02_JPRB * profiles(j) % q(:)    ! - 10% on wv

     ! increments for air surface variables
     prof_inc(j) % s2m % t = -1._JPRB   ! 1k on temperarure
     prof_inc(j) % s2m % q = -0.1_JPRB * profiles(j) % s2m % q ! -10% on 2m wv
     prof_inc(j) % s2m % p = -10._JPRB  ! -10 hpa on pressure
     prof_inc(j) % s2m % u = 0.01_JPRB   ! 0.01_JPRB m/s on wind components
     prof_inc(j) % s2m % v = 0.01_JPRB   ! 0.01_JPRB m/s on wind components
     prof_inc(j) % s2m % wfetc=10._JPRB

     ! increments for skin variables
     prof_inc(j) % skin % surftype = -1  ! no meaning
     prof_inc(j) % skin % t        = -1._JPRB   ! 1k on temperarure
     prof_inc(j) % skin % fastem   = (/-0.01_JPRB,-0.01_JPRB,-0.1_JPRB,-0.001_JPRB,-0.001_JPRB/)

     ! increments for cloud variables
     prof_inc(j) % ctp       = -10._JPRB  ! -10 hpa on pressure
     prof_inc(j) % cfraction = 0.1_JPRB   ! 0.1_JPRB on cloud fraction
  End Do

  ! emissivity
  emissivity_inc(:) = -0.01_JPRB


  Do j = 1, nprofiles
     null_inc(j) % ozone_Data = .False.  ! no meaning
     null_inc(j) % co2_Data   = .False.  ! no meaning
     null_inc(j) % co_Data    = .False.
     null_inc(j) % n2o_Data   = .False.
     null_inc(j) % ch4_Data   = .False.
     if(addaerosl)then 
       null_inc(j) % aer_Data   = .False.
     endif
     if(addclouds)then
       null_inc(j) % cld_Data   = .False.
     endif
     null_inc(j) % clw_Data   = .False.  ! no meaning 
     null_inc(j) % addsolar   = .False.
     null_inc(j) % addrefrac  = .False.
     null_inc(j) % zenangle   = -1  ! no meaning 
     null_inc(j) % azangle    = -1
     null_inc(j) % sunzenangle= -1
     null_inc(j) % sunazangle = -1
     null_inc(j) % latitude   = -1
     null_inc(j) % elevation  = -1
     null_inc(j) % Be  =  -1
     null_inc(j) % cosbk  =  -1
!
     null_inc(j) % p(:)   = 0._JPRB   
     null_inc(j) % t(:)   = 0._JPRB  
     null_inc(j) % q(:)   = 0._JPRB  
     null_inc(j) % o3(:)  = 0._JPRB
     null_inc(j) % co2(:)  = 0._JPRB
     null_inc(j) % co(:)  = 0._JPRB
     null_inc(j) % n2o(:)  = 0._JPRB
     null_inc(j) % ch4(:)  = 0._JPRB
     if(addaerosl)then
       null_inc(j) % aerosols(:,:)= 0._JPRB
     endif
     if(addclouds)then
       null_inc(j) % cloud(:,:)= 0._JPRB
       null_inc(j) % cfrac(:,:)= 0._JPRB
     endif
     null_inc(j) % clw(:) = 0._JPRB
     null_inc(j) % s2m % t  = 0._JPRB
     null_inc(j) % s2m % q  = 0._JPRB
     null_inc(j) % s2m % p  = 0._JPRB
     null_inc(j) % s2m % u  = 0._JPRB
     null_inc(j) % s2m % v  = 0._JPRB
     null_inc(j) % s2m % wfetc=0._JPRB
     null_inc(j) % skin % surftype = -1  ! no meaning 
     null_inc(j) % skin % t        = 0._JPRB
     null_inc(j) % skin % fastem   = (/0._JPRB, 0._JPRB, 0._JPRB, 0._JPRB, 0._JPRB/)
     null_inc(j) % ctp       = 0._JPRB
     null_inc(j) % cfraction = 0._JPRB
!
     prof_inc(j) % Be  = 0._JPRB
     prof_inc(j) % cosbk  = 0._JPRB
  End Do
  null_emissivity_inc(:)  = 0._JPRB

  !..print out increments
  Write(ioout,*)' '
  Write(ioout,*)' input profile increments for tl'
  Write(ioout,444) prof_inc(1) % t(:)
  Write(ioout,444) prof_inc(1) % q(:)  /q_mixratio_to_ppmv
  Write(ioout,444) prof_inc(1) % o3(:) /o3_mixratio_to_ppmv
  Write(ioout,444) prof_inc(1) % co2(:) /co2_mixratio_to_ppmv
  Write(ioout,444) prof_inc(1) % co(:) /co_mixratio_to_ppmv
  Write(ioout,444) prof_inc(1) % n2o(:) /n2o_mixratio_to_ppmv
  Write(ioout,444) prof_inc(1) % ch4(:) /ch4_mixratio_to_ppmv
 ! if(addaerosl)then
 ! Write(ioout,'(a)')'Aerosols 1'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(1,:)
 ! Write(ioout,'(a)')'Aerosols 2'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(2,:)
 ! Write(ioout,'(a)')'Aerosols 3'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(3,:)
 ! Write(ioout,'(a)')'Aerosols 4'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(4,:)
 ! Write(ioout,'(a)')'Aerosols 5'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(5,:)
 ! Write(ioout,'(a)')'Aerosols 6'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(6,:)
 ! Write(ioout,'(a)')'Aerosols 7'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(7,:)
 ! Write(ioout,'(a)')'Aerosols 8'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(8,:)
 ! Write(ioout,'(a)')'Aerosols 9'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(9,:)
 ! Write(ioout,'(a)')'Aerosols 10'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(10,:)
 ! Write(ioout,'(a)')'Aerosols 11'
 ! Write(ioout,'(f15.5)') prof_inc(1) % aerosols(11,:)
 ! Write(ioout,'(a)')'End Aerosols'
 ! endif
 ! if(addclouds)then
 ! Write(ioout,'(a)')'Water cloud 1'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cloud(1,:)
 ! Write(ioout,'(a)')'Water cloud 2'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cloud(2,:)
 ! Write(ioout,'(a)')'Water cloud 3'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cloud(3,:)
 ! Write(ioout,'(a)')'Water cloud 4'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cloud(4,:)
 ! Write(ioout,'(a)')'Water cloud 5'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cloud(5,:)
 ! Write(ioout,'(a)')'Ice cloud'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cloud(6,:)
 ! Write(ioout,'(a)')'End water cloud'
 ! Write(ioout,'(a)')'Fraction water cloud 1'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cfrac(1,:)
 ! Write(ioout,'(a)')'Fraction Water cloud 2'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cfrac(2,:)
 ! Write(ioout,'(a)')'Fraction Water cloud 3'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cfrac(3,:)
 ! Write(ioout,'(a)')'Fraction Water cloud 4'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cfrac(4,:)
 ! Write(ioout,'(a)')'Fraction Water cloud 5'
 ! Write(ioout,'(f15.5)') prof_inc(1) % cfrac(5,:)
 ! Write(ioout,'(a)')'Fraction Ice cloud'    
 ! Write(ioout,'(f15.5)') prof_inc(1) % cfrac(6,:)
 ! Write(ioout,'(a)')'End fraction water cloud'
 ! endif
  Write(ioout,444) prof_inc(1) % clw(:)
  Write(ioout,444)&
         & prof_inc(1) % s2m % t, &
         & prof_inc(1) % s2m % q /q_mixratio_to_ppmv, &
         & prof_inc(1) % s2m % p, &
         & prof_inc(1) % s2m % u, &
         & prof_inc(1) % s2m % v,   &
         & prof_inc(1) % s2m % wfetc 
  Write(ioout,444)&
         & prof_inc(1) % skin % t, &
         & prof_inc(1) % skin % fastem 
  Write(ioout,444)&
         & prof_inc(1) % ctp,&
         & prof_inc(1) % cfraction 
  Write(ioout,*)' '

#ifdef _RTTOV_TSTRAD_TEMP
  call rttov_alloc_temp(                         &
      & temp_status,                             &
      & nprofiles,                               &
      & nchannels,                               &
      & channels,                                &
      & addinterp,                               &
      & profiles(1)%nlevels,                     &
      & coef,                                    &
      & coef_scatt_ir,                           &
      & optp,                                    &
      & addsun .or. Any( profiles(:)%addsolar ), &
      & addaerosl,                               &
      & addclouds,                               &
      & 1_jpim,                                  &
      & temp = temp, temp_tl = temp_tl )     
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Stop       
  EndIf     
#endif




  !
  !...first do profile variables.....................
  Loop1: Do  j =1,jpnav    ! yes clw too!
     Do  ii=1,nlev
        ! initialise all increments to 0
        Do jp = 1, nprofiles
           profiles_tl(jp) % p(:)   = null_inc(jp) % p(:)
           profiles_tl(jp) % t(:)   = null_inc(jp) % t(:)
           profiles_tl(jp) % q(:)   = null_inc(jp) % q(:)
           profiles_tl(jp) % o3(:)  = null_inc(jp) % o3(:)
           profiles_tl(jp) % co2(:) = null_inc(jp) % co2(:)
           profiles_tl(jp) % co(:)  = null_inc(jp) % co(:)
           profiles_tl(jp) % n2o(:) = null_inc(jp) % n2o(:)
           profiles_tl(jp) % ch4(:) = null_inc(jp) % ch4(:)

           if(addaerosl)then
             profiles_tl(jp) % aerosols(:,:)=null_inc(jp) % aerosols(:,:)
           endif

           if(addclouds)then
             profiles_tl(jp) % cloud(:,:)=null_inc(jp) % cloud(:,:)
             profiles_tl(jp) % cfrac(:,:)=null_inc(jp) % cfrac(:,:)
           endif

           profiles_tl(jp) % clw(:) = null_inc(jp) % clw(:)
           profiles_tl(jp) % s2m    = null_inc(jp) % s2m
           profiles_tl(jp) % skin   = null_inc(jp) % skin
           profiles_tl(jp) % ctp    = null_inc(jp) % ctp
           profiles_tl(jp) % cfraction   = null_inc(jp) % cfraction
           profiles_tl(jp) % ozone_Data  = null_inc(jp) % ozone_Data
           profiles_tl(jp) % co2_Data    = null_inc(jp) % co2_Data
           profiles_tl(jp) % co_Data     = null_inc(jp) % co_Data
           profiles_tl(jp) % n2o_Data    = null_inc(jp) % n2o_Data
           profiles_tl(jp) % ch4_Data    = null_inc(jp) % ch4_Data

           if(addaerosl)then
             profiles_tl(jp) % aer_data    = null_inc(jp) % aer_Data 
           endif

           if(addclouds)then
             profiles_tl(jp) % cld_data    = null_inc(jp) % cld_Data
           endif

           profiles_tl(jp) % clw_Data    = null_inc(jp) % clw_Data
           profiles_tl(jp) % addsolar    = null_inc(jp) % addsolar
           profiles_tl(jp) % addrefrac   = null_inc(jp) % addrefrac
           profiles_tl(jp) % zenangle    = null_inc(jp) % zenangle
           profiles_tl(jp) % azangle     = null_inc(jp) % azangle
           profiles_tl(jp) % sunzenangle = null_inc(jp) % sunzenangle
           profiles_tl(jp) % sunazangle  = null_inc(jp) % sunazangle
           profiles_tl(jp) % latitude    = null_inc(jp) % latitude
           profiles_tl(jp) % elevation   = null_inc(jp) % elevation
           profiles_tl(jp) % Be          = null_inc(jp) % Be
           profiles_tl(jp) % cosbk       = null_inc(jp) % cosbk
        End Do
        emissivity_tl(:) = null_emissivity_inc(:)

        Do jp = 1, nprofiles
           ! except the considered level/variable

           if((addaerosl).and.(.NOT.addclouds))then
             if(j>=19.and.j<=30)then
               ixkav(:,j,:)=0_jpim 
               cycle loop1 
             endif 
           else if ((.NOT.addaerosl).and.(addclouds))then
             if(j>=8.and.j<=18)then
               ixkav(:,j,:)=0_jpim
               cycle loop1 
             endif
           else if ((.NOT.addaerosl).and.(.NOT.addclouds))then
             if(j>=8.and.j<=30)then
               ixkav(:,j,:)=0_jpim
               cycle loop1 
             endif
           endif

           Select Case (j)
           Case (1_jpim)
              profiles_tl(jp) % t(ii)   = prof_inc(jp) % t(ii)
              inc_val   = prof_inc(1) % t(ii)
              inc_val_p = inc_val
           Case (2_jpim)
              profiles_tl(jp) % q(ii)   = prof_inc(jp) % q(ii)
              inc_val   = prof_inc(1) % q(ii)
              inc_val_p = inc_val / q_mixratio_to_ppmv
           Case (3_jpim)
              profiles_tl(jp) % o3(ii)  = prof_inc(jp) % o3(ii)
              inc_val   = prof_inc(1) % o3(ii)
              inc_val_p = inc_val / o3_mixratio_to_ppmv
           Case (4_jpim)
              profiles_tl(jp) % co2(ii)  = prof_inc(jp) % co2(ii)
              inc_val   = prof_inc(1) % co2(ii)
              inc_val_p = inc_val / co2_mixratio_to_ppmv
           Case (5_jpim)
              profiles_tl(jp) % n2o(ii)  = prof_inc(jp) % n2o(ii)
              inc_val   = prof_inc(1) % n2o(ii)
              inc_val_p = inc_val / n2o_mixratio_to_ppmv
           Case (6_jpim)
              profiles_tl(jp) % co(ii)  = prof_inc(jp) % co(ii)
              inc_val   = prof_inc(1) % co(ii)
              inc_val_p = inc_val / co_mixratio_to_ppmv
           Case (7_jpim)
              profiles_tl(jp) % ch4(ii)  = prof_inc(jp) % ch4(ii)
              inc_val   = prof_inc(1) % ch4(ii)
              inc_val_p = inc_val / ch4_mixratio_to_ppmv
           Case (8_jpim)
              profiles_tl(jp) % aerosols(1,ii)  = prof_inc(jp) % aerosols(1,ii)
              inc_val   = prof_inc(1) % aerosols(1,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (9_jpim)
              profiles_tl(jp) % aerosols(2,ii)  = prof_inc(jp) % aerosols(2,ii)
              inc_val   = prof_inc(1) % aerosols(2,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (10_jpim)
              profiles_tl(jp) % aerosols(3,ii)  = prof_inc(jp) % aerosols(3,ii)
              inc_val   = prof_inc(1) % aerosols(3,ii)
              inc_val_p = inc_val! !/ ch4_mixratio_to_ppmv
           Case (11_jpim)
              profiles_tl(jp) % aerosols(4,ii)  = prof_inc(jp) % aerosols(4,ii)
              inc_val   = prof_inc(1) % aerosols(4,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (12_jpim)
              profiles_tl(jp) % aerosols(5,ii)  = prof_inc(jp) % aerosols(5,ii)
              inc_val   = prof_inc(1) % aerosols(5,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (13_jpim)
              profiles_tl(jp) % aerosols(6,ii)  = prof_inc(jp) % aerosols(6,ii)
              inc_val   = prof_inc(1) % aerosols(6,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (14_jpim)
              profiles_tl(jp) % aerosols(7,ii)  = prof_inc(jp) % aerosols(7,ii)
              inc_val   = prof_inc(1) % aerosols(7,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (15_jpim)
              profiles_tl(jp) % aerosols(8,ii)  = prof_inc(jp) % aerosols(8,ii)
              inc_val   = prof_inc(1) % aerosols(8,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (16_jpim)
              profiles_tl(jp) % aerosols(9,ii)  = prof_inc(jp) % aerosols(9,ii)
              inc_val   = prof_inc(1) % aerosols(9,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (17_jpim)
              profiles_tl(jp) % aerosols(10,ii)  = prof_inc(jp) % aerosols(10,ii)
              inc_val   = prof_inc(1) % aerosols(10,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (18_jpim)
              profiles_tl(jp) % aerosols(11,ii)  = prof_inc(jp) % aerosols(11,ii)
             inc_val   = prof_inc(1) % aerosols(11,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (19_jpim)
              profiles_tl(jp) % cloud(1,ii)  = prof_inc(jp) % cloud(1,ii)
              inc_val   = prof_inc(1) % cloud(1,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (20_jpim)
              profiles_tl(jp) % cloud(2,ii)  = prof_inc(jp) % cloud(2,ii)
              inc_val   = prof_inc(1) % cloud(2,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (21_jpim)
              profiles_tl(jp) % cloud(3,ii)  = prof_inc(jp) % cloud(3,ii)
              inc_val   = prof_inc(1) % cloud(3,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (22_jpim)
              profiles_tl(jp) % cloud(4,ii)  = prof_inc(jp) % cloud(4,ii)
             inc_val   = prof_inc(1) % cloud(4,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (23_jpim)
              profiles_tl(jp) % cloud(5,ii)  = prof_inc(jp) % cloud(5,ii)
             inc_val   = prof_inc(1) % cloud(5,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (24_jpim)
              profiles_tl(jp) % cloud(6,ii)  = prof_inc(jp) % cloud(6,ii)
             inc_val   = prof_inc(1) % cloud(6,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (25_jpim)
              profiles_tl(jp) % cfrac(1,ii)  = prof_inc(jp) % cfrac(1,ii)
             inc_val   = prof_inc(1) % cfrac(1,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (26_jpim)
              profiles_tl(jp) % cfrac(2,ii)  = prof_inc(jp) % cfrac(2,ii)
             inc_val   = prof_inc(1) % cfrac(2,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (27_jpim)
              profiles_tl(jp) % cfrac(3,ii)  = prof_inc(jp) % cfrac(3,ii)
             inc_val   = prof_inc(1) % cfrac(3,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (28_jpim)
              profiles_tl(jp) % cfrac(4,ii)  = prof_inc(jp) % cfrac(4,ii)
             inc_val   = prof_inc(1) % cfrac(4,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (29_jpim)
              profiles_tl(jp) % cfrac(5,ii)  = prof_inc(jp) % cfrac(5,ii)
             inc_val   = prof_inc(1) % cfrac(5,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (30_jpim)
              profiles_tl(jp) % cfrac(6,ii)  = prof_inc(jp) % cfrac(6,ii)
             inc_val   = prof_inc(1) % cfrac(6,ii)
              inc_val_p = inc_val !/ ch4_mixratio_to_ppmv
           Case (31_jpim) 
              profiles_tl(jp) % clw(ii) = prof_inc(jp) % clw(ii)
              inc_val   = prof_inc(1) % clw(ii)
              inc_val_p = inc_val
           End Select
        End Do

        ! use stored input emmisisvity
        emissivity(:) = input_emissivity(:)

! call-1 rttov_tl
!        if(ii == 1) write(6,*) 'call rttov_tl:   1,nlev   --- jpnav = ', j

        Call rttov_tl( &
              & rttov_errorstatus,  &! out
              & nprofiles,       &! in
              & nchannels,       &! in
              & channels,        &! in
              & lprofiles,       &! in
              & addinterp,       &! in
              & profiles,        &! in
              & profiles_tl,     &! in
              & coef,            &! in
              & coef_scatt_ir,   &
              & optp,            &
              & addsun,          &! in
              & addaerosl,       &
              & addclouds,       &
              & calcemis,        &! in
              & emissivity,      &! inout
              & emissivity_tl,   &! inout
              & transmission,    &! inout
              & transmission_tl, &! inout
              & radiancedata,    &! out
              & radiancedata_tl  &! inout 
#ifdef _RTTOV_TSTRAD_TEMP
              & , temp, temp_tl  &
#endif
              & ) 

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

        If (inc_val == 0._JPRB) Then
           xkbav(ii,j,:)=0._JPRB
           pktav(ii,j,:)=0._JPRB
        Else
           pktav(ii,j,:)=radiancedata_tl%bt(:)
           xkbav(ii,j,:)=radiancedata_tl%bt(:) / inc_val
           If(j==1)Then
              xkradovu(ii,:)=radiancedata_tl%overcast(ii,:)/inc_val
           Endif
        Endif
        If (inc_val_p .eq. 0.0) Then                  !
              inc_val   = prof_inc(1) % clw(ii)       !###Nasty fix for now
              inc_val_p = inc_val                     !
        Endif
        ixkav(ii,j,:)=Nint(radiancedata_tl%bt(:)*facpav(ii,j)/ inc_val_p)
     End Do
  End Do Loop1

  !          ... and print it.
  Write (ioout,*)' '
  Write (ioout,*)'k-matrix: tangent linear.'
  Write (ioout,*)' '
  If (addclouds .or. addaerosl)Then
    nprint = jpnav 
  Else
    nprint = 4
  Endif
  Do j = 1 , nprint                   ! lwp on
     Write (ioout,'(a30)')title(j)
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Do i = 1 , nlev
        Write (ioout,333)i,(ixkav(i,j,jch),jch=1,nchan_out)
     Enddo
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Write (ioout,*)' '
  Enddo


  ! optionally print full radiance arrays
   If(addclouds)Then
     Write (ioout,*)' '
     Write (ioout,*)'k-matrix: upwelling radiance tl '
     Do  jch=1,nchan_out
        ixkov(:,jch)=Nint(xkradovu(:,jch)*facovu(:))
     End Do
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Do i = 1 , nlev
        Write (ioout,333)i,(ixkov(i,jch),jch=1,nchan_out)
     Enddo
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
   Endif
  ! initialise all increments to 0
  Do jp = 1, nprofiles
     profiles_tl(jp) % p(:)   = null_inc(jp) % p(:)
     profiles_tl(jp) % t(:)   = null_inc(jp) % t(:)
     profiles_tl(jp) % q(:)   = null_inc(jp) % q(:)
     profiles_tl(jp) % o3(:)  = null_inc(jp) % o3(:)
     profiles_tl(jp) % co2(:) = null_inc(jp) % co2(:)
     profiles_tl(jp) % co(:)  = null_inc(jp) % co(:)
     profiles_tl(jp) % n2o(:) = null_inc(jp) % n2o(:)
     profiles_tl(jp) % ch4(:) = null_inc(jp) % ch4(:)
     if(addaerosl)then
       profiles_tl(jp) % aerosols(:,:)= null_inc(jp) % aerosols(:,:)
     endif
     if(addclouds)then
       profiles_tl(jp) % cloud(:,:)= null_inc(jp) % cloud(:,:)
       profiles_tl(jp) % cfrac(:,:)= null_inc(jp) % cfrac(:,:)
     endif
     profiles_tl(jp) % clw(:) = null_inc(jp) % clw(:)
     profiles_tl(jp) % s2m    = null_inc(jp) % s2m
     profiles_tl(jp) % skin   = null_inc(jp) % skin
     profiles_tl(jp) % ctp    = null_inc(jp) % ctp
     profiles_tl(jp) % cfraction   = null_inc(jp) % cfraction
     profiles_tl(jp) % ozone_Data  = null_inc(jp) % ozone_Data
     profiles_tl(jp) % co2_Data    = null_inc(jp) % co2_Data
     profiles_tl(jp) % co_Data     = null_inc(jp) % co_Data
     profiles_tl(jp) % n2o_Data    = null_inc(jp) % n2o_Data
     profiles_tl(jp) % ch4_Data    = null_inc(jp) % ch4_Data
     if(addaerosl)then
       profiles_tl(jp) % aer_Data    = null_inc(jp) % aer_Data
     endif
     if(addclouds)then
       profiles_tl(jp) % cld_Data    = null_inc(jp) % cld_Data
     endif
     profiles_tl(jp) % clw_Data    = null_inc(jp) % clw_Data
     profiles_tl(jp) % addsolar    = null_inc(jp) % addsolar
     profiles_tl(jp) % addrefrac   = null_inc(jp) % addrefrac
     profiles_tl(jp) % zenangle    = null_inc(jp) % zenangle   
     profiles_tl(jp) % azangle     = null_inc(jp) % azangle 
     profiles_tl(jp) % sunzenangle = null_inc(jp) % sunzenangle
     profiles_tl(jp) % sunazangle  = null_inc(jp) % sunazangle
     profiles_tl(jp) % latitude    = null_inc(jp) % latitude
     profiles_tl(jp) % elevation   = null_inc(jp) % elevation
     profiles_tl(jp) % Be          = null_inc(jp) % Be
     profiles_tl(jp) % cosbk       = null_inc(jp) % cosbk
  End Do
  emissivity_tl(:) = null_emissivity_inc(:)

  !.......now do surface, skin and cloud variables
  Do  j =1,sscvar

     Do jp = 1, nprofiles
        ! except the considered level/variable
        Select Case (j)
        Case (1_jpim)
           ! t 2m
           inc_val = prof_inc(jp) % s2m % t
           inc_val_p = inc_val 
           profiles_tl(jp) % s2m % t   = inc_val
        Case (2_jpim)
           ! wv 2m
           inc_val = prof_inc(jp) % s2m % q
           inc_val_p = inc_val / q_mixratio_to_ppmv
           profiles_tl(jp) % s2m % q   = inc_val
        Case (3_jpim)
           ! surface pressure
           inc_val = prof_inc(jp) % s2m % p
           inc_val_p = inc_val 
           profiles_tl(jp) % s2m % p   = inc_val
        Case (4_jpim)
           ! wind speed u component
           inc_val = prof_inc(jp) % s2m % u
           inc_val_p = inc_val 
           profiles_tl(jp) % s2m % u   = inc_val
        Case (5_jpim)
           ! wind speed v component
           inc_val = prof_inc(jp) % s2m % v
           inc_val_p = inc_val 
           profiles_tl(jp) % s2m % v   = inc_val
        Case (6_jpim)
           ! wind fetc    component
           inc_val = prof_inc(jp) % s2m % wfetc 
           profiles_tl(jp) % s2m % wfetc   = inc_val
        Case (7_jpim)
           ! skin temp
           inc_val = prof_inc(jp) % skin % t
           inc_val_p = inc_val 
           profiles_tl(jp) % skin % t   = inc_val
        Case (8_jpim)
           ! fastem land coef 1
           inc_val = prof_inc(jp) % skin % fastem(1)
           inc_val_p = inc_val 
           profiles_tl(jp) % skin % fastem(1)   = inc_val
        Case (9_jpim)
           ! fastem land coef 2
           inc_val = prof_inc(jp) % skin % fastem(2)
           inc_val_p = inc_val 
           profiles_tl(jp) % skin % fastem(2)   = inc_val
        Case (10_jpim)
           ! fastem land coef 3
           inc_val = prof_inc(jp) % skin % fastem(3)
           inc_val_p = inc_val 
           profiles_tl(jp) % skin % fastem(3)   = inc_val
        Case (11_jpim)
           ! fastem land coef 4
           inc_val = prof_inc(jp) % skin % fastem(4)
           inc_val_p = inc_val 
           profiles_tl(jp) % skin % fastem(4)   = inc_val
        Case (12_jpim)
           ! fastem land coef 5
           inc_val = prof_inc(jp) % skin % fastem(5)
           inc_val_p = inc_val 
           profiles_tl(jp) % skin % fastem(5)   = inc_val
        Case (13_jpim)
           ! cloud top pressure
           inc_val = prof_inc(jp) % ctp
           inc_val_p = inc_val 
           profiles_tl(jp) % ctp   = inc_val
        Case (14_jpim)
           ! cloud fraction
           inc_val = prof_inc(jp) % cfraction
           inc_val_p = inc_val 
           profiles_tl(jp) % cfraction   = inc_val
        End Select
     End Do

     ! use stored input emmisisvity
     emissivity(:) = input_emissivity(:)
     emissivity_tl(:) = null_emissivity_inc(:)

! call-2 of rttov_tl
!   if(j == 1) write(6,*) 'call rttov_tl:   1,sscvar'

     Call rttov_tl( &
           & rttov_errorstatus,  &! out
           & nprofiles,       &! in
           & nchannels,       &! in
           & channels,        &! in
           & lprofiles,       &! in
           & addinterp,       &! in
           & profiles,        &! in
           & profiles_tl,     &! in
           & coef,            &! in
           & coef_scatt_ir,   &
           & optp,            &
           & addsun,          &! in
           & addaerosl,       &
           & addclouds,            &
           & calcemis,        &! in
           & emissivity,      &! inout
           & emissivity_tl,   &! inout
           & transmission,    &! inout
           & transmission_tl, &! inout
           & radiancedata,    &! inout
           & radiancedata_tl  &! inout 
#ifdef _RTTOV_TSTRAD_TEMP
           & , temp, temp_tl  &
#endif
           & ) 

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

     Do jp = 1, nprofiles
        ! reset profile for next variable
        Select Case (j)
        Case (1_jpim)
           ! t 2m
           profiles_tl(jp) % s2m % t   = null_inc(jp) % s2m % t
        Case (2_jpim)
           ! wv 2m
           profiles_tl(jp) % s2m % q   = null_inc(jp) % s2m % q
        Case (3_jpim)
           ! surface pressure
           profiles_tl(jp) % s2m % p   = null_inc(jp) % s2m % p
        Case (4_jpim)
           ! wind speed u component
           profiles_tl(jp) % s2m % u   = null_inc(jp) % s2m % u
        Case (5_jpim)
           ! wind speed v component
           profiles_tl(jp) % s2m % v   = null_inc(jp) % s2m % v
        Case (6_jpim)
           ! wind fetc component
           profiles_tl(jp) % s2m % wfetc   = null_inc(jp) % s2m % wfetc 
        Case (7_jpim)
           ! skin temp
           profiles_tl(jp) % skin % t   = null_inc(jp) % skin % t
        Case (8_jpim)
           ! fastem land coef 1
           profiles_tl(jp) % skin % fastem(1)   = null_inc(jp) % skin % fastem(1)
        Case (9_jpim)
           ! fastem land coef 2
           profiles_tl(jp) % skin % fastem(2)   = null_inc(jp) % skin % fastem(2)
        Case (10_jpim)
           ! fastem land coef 3
           profiles_tl(jp) % skin % fastem(3)   = null_inc(jp) % skin % fastem(3)
        Case (11_jpim)
           ! fastem land coef 4
           profiles_tl(jp) % skin % fastem(4)   = null_inc(jp) % skin % fastem(4)
        Case (12_jpim)
           ! fastem land coef 5
           profiles_tl(jp) % skin % fastem(5)   = null_inc(jp) % skin % fastem(5)
        Case (13_jpim)
           ! cloud top pressure
           profiles_tl(jp) % ctp   = null_inc(jp) % ctp
        Case (14_jpim)
           ! cloud fraction
           profiles_tl(jp) % cfraction   = null_inc(jp) % cfraction
        End Select
     End Do

     If( inc_val == 0._JPRB ) Then
        pktsav(j,:) = 0._JPRB
        xkbsav(j,:) = 0._JPRB
     Else
        pktsav(j,:) = radiancedata_tl%bt(:)
        xkbsav(j,:) = radiancedata_tl%bt(:) / inc_val
     End If
     ixksav(j,:) = Nint(radiancedata_tl%bt(:)*facsav(j) / inc_val_p)

!if(j==6)stop

  End Do
  !          ... and print it.
  Write (ioout,*)' surface variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = 1 , jpnsav
     Write (ioout,333)i,(ixksav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' skin variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = jpnsav+1 , jpnsav+jpnssv
     Write (ioout,333)i-jpnsav,(ixksav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' cloud variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = jpnsav+jpnssv+1 , sscvar
     Write (ioout,333)i-jpnsav-jpnssv,(ixksav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  !.......now do surface emissivity
  Do  j =1,nchannels
     ! use stored input emmisisvity
     emissivity(:)    = input_emissivity(:)

     ! increment for only one channel
     emissivity_tl(:) = null_emissivity_inc(:)
        inc_val          = emissivity_inc(J)
        emissivity_tl(j) = inc_val

! call-3 of rttov_tl
!     if(j == 1) write(6,*) 'call rttov_tl:   1,nbtout'

    Call rttov_tl( &
           & rttov_errorstatus,  &! out
           & nprofiles,       &! in
           & nchannels,       &! in
           & channels,        &! in
           & lprofiles,       &! in
           & addinterp,       &! in
           & profiles,        &! in
           & profiles_tl,     &! in
           & coef,            &! in
           & coef_scatt_ir,   &
           & optp,            &
           & addsun,          &! in
           & addaerosl,       &
           & addclouds,       &
           & calcemis,        &! in
           & emissivity,      &! inout
           & emissivity_tl,   &! inout
           & transmission,    &! inout
           & transmission_tl, &! inout
           & radiancedata,    &! inout
           & radiancedata_tl  &! inout 
#ifdef _RTTOV_TSTRAD_TEMP
           & , temp, temp_tl  &
#endif
           & ) 

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

     !.......Except for FASTEM-2 (input <0)
     !If( emissivity_tl(j) == 0. .Or. input_emissivity(j)<0.) Then
    If( emissivity_tl(j) == 0._JPRB .Or.( coef % fastem_ver >= 2 .and. calcemis(j)) ) Then 
        xkbem(j) = 0._JPRB
     Else
        xkbem(j) = radiancedata_tl%bt(j) / inc_val
     End If

  End Do
  ixkem(:)=Nint(xkbem(:)*facem)

  Write (ioout,*)' surface emissivity '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do jp = 1, nprofiles
     joff = (jp-1) * nchan_out
     Write (ioout,333)jp,(ixkem(jch+joff),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  !
  ! Now compute Brute force matrix
  !
  !save forward model radiances and emissivities
  ! do not consider overcast radiances
  radiance_fwd % clear(:)    = radiancedata % clear(:)
  radiance_fwd % cloudy(:)   = radiancedata % cloudy(:)
  radiance_fwd % bt(:)       = radiancedata % bt(:)
  emissivity_fwd(:)          = emissivity(:)





  Loop2: Do j =1,jpnav    ! yes clw too!
     Do  ii=1,nlev

        ! initialise profile with input profile
        Do jp = 1, nprofiles
           profiles_bf(jp) % p(:)   = profiles(jp) % p(:)
           profiles_bf(jp) % t(:)   = profiles(jp) % t(:)
           profiles_bf(jp) % q(:)   = profiles(jp) % q(:)
           profiles_bf(jp) % o3(:)  = profiles(jp) % o3(:)
           profiles_bf(jp) % co2(:) = profiles(jp) % co2(:)
           profiles_bf(jp) % co(:)  = profiles(jp) % co(:)
           profiles_bf(jp) % n2o(:) = profiles(jp) % n2o(:)
           profiles_bf(jp) % ch4(:) = profiles(jp) % ch4(:)

           if(addaerosl)then
             profiles_bf(jp) % aerosols(:,:)= profiles(jp) % aerosols(:,:)
           endif

           if(addclouds)then
             profiles_bf(jp) % cloud(:,:)= profiles(jp) % cloud(:,:)
             profiles_bf(jp) % cfrac(:,:)= profiles(jp) % cfrac(:,:)
             profiles_bf(jp) % idg       = profiles(jp) % idg
             profiles_bf(jp) % ish       = profiles(jp) % ish
           endif

           profiles_bf(jp) % clw(:) = profiles(jp) % clw(:)
           profiles_bf(jp) % s2m    = profiles(jp) % s2m
           profiles_bf(jp) % skin   = profiles(jp) % skin
           profiles_bf(jp) % ctp    = profiles(jp) % ctp
           profiles_bf(jp) % cfraction   = profiles(jp) % cfraction
           profiles_bf(jp) % ozone_Data  = profiles(jp) % ozone_Data
           profiles_bf(jp) % co2_Data    = profiles(jp) % co2_Data
           profiles_bf(jp) % co_Data     = profiles(jp) % co_Data
           profiles_bf(jp) % n2o_Data    = profiles(jp) % n2o_Data
           profiles_bf(jp) % ch4_Data    = profiles(jp) % ch4_Data
           if(addaerosl)then
             profiles_bf(jp) % aer_data    = profiles(jp) % aer_Data
           endif
           if(addclouds)then
             profiles_bf(jp) % cld_data    = profiles(jp) % cld_Data
           endif
           profiles_bf(jp) % clw_Data    = profiles(jp) % clw_Data
           profiles_bf(jp) % addsolar    = profiles(jp) % addsolar
           profiles_bf(jp) % addrefrac   = profiles(jp) % addrefrac
           profiles_bf(jp) % zenangle    = profiles(jp) % zenangle
           profiles_bf(jp) % azangle     = profiles(jp) % azangle
           profiles_bf(jp) % sunzenangle = profiles(jp) % sunzenangle
           profiles_bf(jp) % sunazangle  = profiles(jp) % sunazangle
           profiles_bf(jp) % latitude    = profiles(jp) % latitude 
           profiles_bf(jp) % elevation   = profiles(jp) % elevation
           profiles_bf(jp) % Be          = profiles(jp) % Be
           profiles_bf(jp) % cosbk       = profiles(jp) % cosbk
        End Do

        Do jp = 1, nprofiles
           ! except the considered level/variable

           if((addaerosl).and.(.NOT.addclouds))then
             if(j>=19.and.j<=30)then
               xktav(:,j,:)=0._jprb
               cycle loop2
             endif
           else if ((.NOT.addaerosl).and.(addclouds))then
             if(j>=8.and.j<=18)then
               xktav(:,j,:)=0._jprb
               cycle loop2 
             endif
           else if ((.NOT.addaerosl).and.(.NOT.addclouds))then
             if(j>=8.and.j<=30)then
               xktav(:,j,:)=0._jprb
               cycle loop2 
             endif
           endif

           Select Case (j)
           Case (1_jpim)
              inc_val = prof_inc(1) % t(ii)
              profiles_bf(jp) % t(ii) = profiles_bf(jp) % t(ii) + inc_val
           Case (2_jpim)
              inc_val = prof_inc(1) % q(ii)
              profiles_bf(jp) % q(ii)   = profiles_bf(jp) % q(ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % q(ii)/q_mixratio_to_ppmv
           Case (3_jpim)
              inc_val = prof_inc(1) % o3(ii)
              profiles_bf(jp) % o3(ii)  = profiles_bf(jp) % o3(ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % o3(ii)/o3_mixratio_to_ppmv
           Case (4_jpim)
              inc_val = prof_inc(1) % co2(ii)
              profiles_bf(jp) % co2(ii)  = profiles_bf(jp) % co2(ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % co2(ii)/co2_mixratio_to_ppmv
           Case (5_jpim)
              inc_val = prof_inc(1) % n2o(ii)
              profiles_bf(jp) % n2o(ii)  = profiles_bf(jp) % n2o(ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % n2o(ii)/n2o_mixratio_to_ppmv
           Case (6_jpim)
              inc_val = prof_inc(1) % co(ii)
              profiles_bf(jp) % co(ii)  = profiles_bf(jp) % co(ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % co(ii)/co_mixratio_to_ppmv
           Case (7_jpim)
              inc_val = prof_inc(1) % ch4(ii)
              profiles_bf(jp) % ch4(ii)  = profiles_bf(jp) % ch4(ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % ch4(ii)/ch4_mixratio_to_ppmv
           Case (8_jpim)
             inc_val = prof_inc(1) % aerosols(1,ii)
              profiles_bf(jp) % aerosols(1,ii)  = profiles_bf(jp) % aerosols(1,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(1,ii)!/ch4_mixratio_to_ppmv
           Case (9_jpim)
              inc_val = prof_inc(1) % aerosols(2,ii)
              profiles_bf(jp) % aerosols(2,ii)  = profiles_bf(jp) % aerosols(2,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(2,ii)!/ch4_mixratio_to_ppmv
           Case (10_jpim)
              inc_val = prof_inc(1) % aerosols(3,ii)
              profiles_bf(jp) % aerosols(3,ii)  = profiles_bf(jp) % aerosols(3,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(3,ii)!/ch4_mixratio_to_ppmv
           Case (11_jpim)
              inc_val = prof_inc(1) % aerosols(4,ii)
              profiles_bf(jp) % aerosols(4,ii)  = profiles_bf(jp) % aerosols(4,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(4,ii)!/ch4_mixratio_to_ppmv
           Case (12_jpim)
              inc_val = prof_inc(1) % aerosols(5,ii)
              profiles_bf(jp) % aerosols(5,ii)  = profiles_bf(jp) % aerosols(5,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(5,ii)!/ch4_mixratio_to_ppmv
           Case (13_jpim)
              inc_val = prof_inc(1) % aerosols(6,ii)
              profiles_bf(jp) % aerosols(6,ii)  = profiles_bf(jp) % aerosols(6,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(6,ii)!/ch4_mixratio_to_ppmv
           Case (14_jpim)
              inc_val = prof_inc(1) % aerosols(7,ii)
              profiles_bf(jp) % aerosols(7,ii)  = profiles_bf(jp) % aerosols(7,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(7,ii)!/ch4_mixratio_to_ppmv
           Case (15_jpim)
              inc_val = prof_inc(1) % aerosols(8,ii)
              profiles_bf(jp) % aerosols(8,ii)  = profiles_bf(jp) % aerosols(8,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(8,ii)!/ch4_mixratio_to_ppmv
           Case (16_jpim)
              inc_val = prof_inc(1) % aerosols(9,ii)
              profiles_bf(jp) % aerosols(9,ii)  = profiles_bf(jp) % aerosols(9,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(9,ii)!/ch4_mixratio_to_ppmv
           Case (17_jpim)
              inc_val = prof_inc(1) % aerosols(10,ii)
              profiles_bf(jp) % aerosols(10,ii)  = profiles_bf(jp) % aerosols(10,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(10,ii)!/ch4_mixratio_to_ppmv
           Case (18_jpim)
              inc_val = prof_inc(1) % aerosols(11,ii)
              profiles_bf(jp) % aerosols(11,ii)  = profiles_bf(jp) % aerosols(11,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % aerosols(11,ii)!/ch4_mixratio_to_ppmv
           Case (19_jpim)
              inc_val = prof_inc(1) % cloud(1,ii)
              profiles_bf(jp) % cloud(1,ii)  = profiles_bf(jp) % cloud(1,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cloud(1,ii)!/ch4_mixratio_to_ppmv
           Case (20_jpim)
              inc_val = prof_inc(1) % cloud(2,ii)
              profiles_bf(jp) % cloud(2,ii)  = profiles_bf(jp) % cloud(2,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cloud(2,ii)!/ch4_mixratio_to_ppmv
           Case (21_jpim)
              inc_val = prof_inc(1) % cloud(3,ii)
              profiles_bf(jp) % cloud(3,ii)  = profiles_bf(jp) % cloud(3,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cloud(3,ii)!/ch4_mixratio_to_ppmv
           Case (22_jpim)
              inc_val = prof_inc(1) % cloud(4,ii)
              profiles_bf(jp) % cloud(4,ii)  = profiles_bf(jp) % cloud(4,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cloud(4,ii)!/ch4_mixratio_to_ppmv
           Case (23_jpim)
              inc_val = prof_inc(1) % cloud(5,ii)
              profiles_bf(jp) % cloud(5,ii)  = profiles_bf(jp) % cloud(5,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cloud(5,ii)!/ch4_mixratio_to_ppmv
           Case (24_jpim)
              inc_val = prof_inc(1) % cloud(6,ii)
              profiles_bf(jp) % cloud(6,ii)  = profiles_bf(jp) % cloud(6,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cloud(6,ii)!/ch4_mixratio_to_ppmv
           Case (25_jpim)
              inc_val = prof_inc(1) % cfrac(1,ii)
              profiles_bf(jp) % cfrac(1,ii)  = profiles_bf(jp) % cfrac(1,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cfrac(1,ii)!/ch4_mixratio_to_ppmv
           Case (26_jpim)
              inc_val = prof_inc(1) % cfrac(2,ii)
              profiles_bf(jp) % cfrac(2,ii)  = profiles_bf(jp) % cfrac(2,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cfrac(2,ii)!/ch4_mixratio_to_ppmv
           Case (27_jpim)
              inc_val = prof_inc(1) % cfrac(3,ii)
              profiles_bf(jp) % cfrac(3,ii)  = profiles_bf(jp) % cfrac(3,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cfrac(3,ii)!/ch4_mixratio_to_ppmv
           Case (28_jpim)
              inc_val = prof_inc(1) % cfrac(4,ii)
              profiles_bf(jp) % cfrac(4,ii)  = profiles_bf(jp) % cfrac(4,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cfrac(4,ii)!/ch4_mixratio_to_ppmv
           Case (29_jpim)
              inc_val = prof_inc(1) % cfrac(5,ii)
              profiles_bf(jp) % cfrac(5,ii)  = profiles_bf(jp) % cfrac(5,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cfrac(5,ii)!/ch4_mixratio_to_ppmv
           Case (30_jpim)
              inc_val = prof_inc(1) % cfrac(6,ii)
              profiles_bf(jp) % cfrac(6,ii)  = profiles_bf(jp) % cfrac(6,ii) + inc_val
              ! change increment for output compatibilty
              inc_val = prof_inc(1) % cfrac(6,ii)!/ch4_mixratio_to_ppmv
           Case (31_jpim) 
              inc_val = prof_inc(1) % clw(ii)
              profiles_bf(jp) % clw(ii) = profiles_bf(jp) % clw(ii) + inc_val
           End Select
        End Do

        ! use stored input emmisisvity
        emissivity(:) = input_emissivity(:)

! call-1 of rttov_direct
!        if(ii == 1) write(6,*) 'call rttov_direct (brute-force):   1,nlev   --- jpnav = ', j

        Call rttov_direct( &
              & rttov_errorstatus,  &! out
              & nprofiles,     &! in
              & nchannels,     &! in
              & channels,      &! in
              & lprofiles,     &! in
              & addinterp,     &! in
              & profiles_bf,   &! in
              & coef,          &! in
              & coef_scatt_ir, &
              & optp,          &
              & addsun,        &
              & addaerosl,     &! in
              & addclouds,     &! in
              & calcemis,      &! in
              & emissivity,    &! inout
              & transmission,  &! out
              & radiancedata   &! inout 
#ifdef _RTTOV_TSTRAD_TEMP         
              & , temp         &
#endif
        )
        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

        If (inc_val == 0._JPRB) Then
           xktav(ii,j,:)=0._JPRB
        Else
           xktav(ii,j,:)=( radiancedata%bt(:) - radiance_fwd%bt(:) )/ inc_val
        Endif

     End Do
  Enddo loop2






  Do  jch=1,nchannels
     ixkdav(:,:,jch)=Nint(xktav(:,:,jch)*facpav(:,:))
  End Do

  ! initialise profile with input profile
  Do jp = 1, nprofiles
     profiles_bf(jp) % p(:)   = profiles(jp) % p(:)
     profiles_bf(jp) % t(:)   = profiles(jp) % t(:)
     profiles_bf(jp) % q(:)   = profiles(jp) % q(:)
     profiles_bf(jp) % o3(:)  = profiles(jp) % o3(:)
     profiles_bf(jp) % co2(:) = profiles(jp) % co2(:)
     profiles_bf(jp) % co(:)  = profiles(jp) % co(:)
     profiles_bf(jp) % n2o(:) = profiles(jp) % n2o(:)
     profiles_bf(jp) % ch4(:) = profiles(jp) % ch4(:)
     if(addaerosl)then
       profiles_bf(jp) % aerosols(:,:)=profiles(jp) % aerosols(:,:)
     endif
     if(addclouds)then
       profiles_bf(jp) % cloud(:,:)=profiles(jp) % cloud(:,:)
       profiles_bf(jp) % cfrac(:,:)=profiles(jp) % cfrac(:,:)
     endif
     profiles_bf(jp) % clw(:) = profiles(jp) % clw(:)
     profiles_bf(jp) % s2m    = profiles(jp) % s2m
     profiles_bf(jp) % skin   = profiles(jp) % skin
     profiles_bf(jp) % ctp    = profiles(jp) % ctp
     profiles_bf(jp) % cfraction   = profiles(jp) % cfraction
     profiles_bf(jp) % ozone_Data  = profiles(jp) % ozone_Data
     profiles_bf(jp) % co2_Data    = profiles(jp) % co2_Data
     profiles_bf(jp) % co_Data     = profiles(jp) % co_Data
     profiles_bf(jp) % n2o_Data    = profiles(jp) % n2o_Data
     profiles_bf(jp) % ch4_Data    = profiles(jp) % ch4_Data
     if(addaerosl)then
       profiles_bf(jp) % aer_Data    = profiles(jp) % aer_Data
     endif
     if(addclouds)then
     profiles_bf(jp) % cld_Data    = profiles(jp) % cld_Data
     endif
     profiles_bf(jp) % clw_Data    = profiles(jp) % clw_Data
     profiles_bf(jp) % addsolar    = profiles(jp) % addsolar
     profiles_bf(jp) % addrefrac   = profiles(jp) % addrefrac
     profiles_bf(jp) % zenangle    = profiles(jp) % zenangle
     profiles_bf(jp) % azangle     = profiles(jp) % azangle
     profiles_bf(jp) % sunzenangle = profiles(jp) % sunzenangle
     profiles_bf(jp) % sunazangle  = profiles(jp) % sunazangle
     profiles_bf(jp) % latitude    = profiles(jp) % latitude
     profiles_bf(jp) % elevation   = profiles(jp) % elevation
     profiles_bf(jp) % Be          = profiles(jp) % Be
     profiles_bf(jp) % cosbk       = profiles(jp) % cosbk
  End Do

  !.......now do surface, skin and cloud variables
  Do  j =1,sscvar
     Do jp = 1, nprofiles
        ! except the considered level/variable
        Select Case (j)
        Case (1_jpim)
           ! t 2m
           inc_val = prof_inc(jp) % s2m % t
           profiles_bf(jp) % s2m % t   = profiles_bf(jp) % s2m % t + inc_val
        Case (2_jpim)
           ! wv 2m
           inc_val = prof_inc(jp) % s2m % q
           profiles_bf(jp) % s2m % q   = profiles_bf(jp) % s2m % q + inc_val
           inc_val = inc_val / q_mixratio_to_ppmv
        Case (3_jpim)
           ! surface pressure
           inc_val = prof_inc(jp) % s2m % p
           profiles_bf(jp) % s2m % p   = profiles_bf(jp) % s2m % p + inc_val
        Case (4_jpim)
           ! wind speed u component
           inc_val = prof_inc(jp) % s2m % u
           profiles_bf(jp) % s2m % u   = profiles_bf(jp) % s2m % u + inc_val
        Case (5_jpim)
           ! wind speed v component
           inc_val = prof_inc(jp) % s2m % v
           profiles_bf(jp) % s2m % v   = profiles_bf(jp) % s2m % v + inc_val
        Case (6_jpim)
           ! wind fetc component
           inc_val = prof_inc(jp) % s2m % wfetc 
           profiles_bf(jp) % s2m % wfetc   = profiles_bf(jp) % s2m % wfetc + inc_val
        Case (7_jpim)
           ! skin temp
           inc_val = prof_inc(jp) % skin % t
           profiles_bf(jp) % skin % t   = profiles_bf(jp) % skin % t + inc_val
        Case (8_jpim)
           ! fastem land coef 1
           inc_val = prof_inc(jp) % skin % fastem(1)
           profiles_bf(jp) % skin % fastem(1)   = profiles_bf(jp) % skin % fastem(1) + inc_val
        Case (9_jpim)
           ! fastem land coef 2
           inc_val = prof_inc(jp) % skin % fastem(2)
           profiles_bf(jp) % skin % fastem(2)   = profiles_bf(jp) % skin % fastem(2) + inc_val
        Case (10_jpim)
           ! fastem land coef 3
           inc_val = prof_inc(jp) % skin % fastem(3)
           profiles_bf(jp) % skin % fastem(3)   = profiles_bf(jp) % skin % fastem(3) + inc_val
        Case (11_jpim)
           ! fastem land coef 4
           inc_val = prof_inc(jp) % skin % fastem(4)
           profiles_bf(jp) % skin % fastem(4)   = profiles_bf(jp) % skin % fastem(4) + inc_val
        Case (12_jpim)
           ! fastem land coef 5
           inc_val = prof_inc(jp) % skin % fastem(5)
           profiles_bf(jp) % skin % fastem(5)   = profiles_bf(jp) % skin % fastem(5) + inc_val
        Case (13_jpim)
           ! cloud top pressure
           inc_val = prof_inc(jp) % ctp
           profiles_bf(jp) % ctp   = profiles_bf(jp) % ctp + inc_val
        Case (14_jpim)
           ! cloud fraction
           inc_val = prof_inc(jp) % cfraction
           profiles_bf(jp) % cfraction   = profiles_bf(jp) % cfraction + inc_val
        End Select
     End Do

     ! use stored input emmisisvity
     emissivity(:) = input_emissivity(:)

! call-2 of rttov_direct
!     if(j == 1) write(6,*) 'call rttov_direct (brute-force):   1,sscvar'

     Call rttov_direct( &
           & rttov_errorstatus,  &! out
           & nprofiles,      &! in
           & nchannels,      &! in
           & channels,       &! in
           & lprofiles,      &! in
           & addinterp,      &! in
           & profiles_bf,    &! in
           & coef,           &! in
           & coef_scatt_ir,  &! in
           & optp,           &! in
           & addsun,         &
           & addaerosl,      &! in
           & addclouds,      &! in
           & calcemis,       &! in
           & emissivity,     &! inout
           & transmission,   &! out
           & radiancedata    &! inout 
#ifdef _RTTOV_TSTRAD_TEMP         
           & , temp          &
#endif
        )

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

     Do jp = 1, nprofiles
        ! reset profile for next variable
        Select Case (j)
        Case (1_jpim)
           ! t 2m
           profiles_bf(jp) % s2m % t   = profiles(jp) % s2m % t
        Case (2_jpim)
           ! wv 2m
           profiles_bf(jp) % s2m % q   = profiles(jp) % s2m % q
        Case (3_jpim)
           ! surface pressure
           profiles_bf(jp) % s2m % p   = profiles(jp) % s2m % p
        Case (4_jpim)
           ! wind speed u component
           profiles_bf(jp) % s2m % u   = profiles(jp) % s2m % u
        Case (5_jpim)
           ! wind speed v component
           profiles_bf(jp) % s2m % v   = profiles(jp) % s2m % v
        Case (6_jpim)
           ! wind fetch component
           profiles_bf(jp) % s2m % wfetc   = profiles(jp) % s2m % wfetc 
        Case (7_jpim)
           ! skin temp
           profiles_bf(jp) % skin % t   = profiles(jp) % skin % t
        Case (8_jpim)
           ! fastem land coef 1
           profiles_bf(jp) % skin % fastem(1)   = profiles(jp) % skin % fastem(1)
        Case (9_jpim)
           ! fastem land coef 2
           profiles_bf(jp) % skin % fastem(2)   = profiles(jp) % skin % fastem(2)
        Case (10_jpim)
           ! fastem land coef 3
           profiles_bf(jp) % skin % fastem(3)   = profiles(jp) % skin % fastem(3)
        Case (11_jpim)
           ! fastem land coef 4
           profiles_bf(jp) % skin % fastem(4)   = profiles(jp) % skin % fastem(4)
        Case (12_jpim)
           ! fastem land coef 5
           profiles_bf(jp) % skin % fastem(5)   = profiles(jp) % skin % fastem(5)
        Case (13_jpim)
           ! cloud top pressure
           profiles_bf(jp) % ctp   = profiles(jp) % ctp
        Case (14_jpim)
           ! cloud fraction
           profiles_bf(jp) % cfraction   = profiles(jp) % cfraction
        End Select
     End Do

     If( inc_val == 0._JPRB ) Then
        xktsav(j,:) = 0._JPRB
     Else
        xktsav(j,:)=( radiancedata%bt(:) - radiance_fwd%bt(:) )/ inc_val
     End If

  End Do





  Do  jch=1,nchannels
     ixkdsav(:,jch)=Nint(xktsav(:,jch)*facsav(:))
  End Do

  !.......now do surface emissivity
  Do  j =1,nchannels

     ! increment for only one channel
     ! use emissivity from FWD model output
     emissivity_bf(:) = emissivity_fwd(:)
     calcemis_bf(:)   = .False.
     inc_val  = emissivity_inc(j)
     emissivity_bf(j) = emissivity_bf(j) + inc_val 

! call-3 of rttov_direct
!     if(j == 1) write(6,*) 'call rttov_direct (brute-force):   1, nbtout'

     Call rttov_direct( &
           & rttov_errorstatus,  &! out
           & nprofiles,          &! in
           & nchannels,          &! in
           & channels,           &! in
           & lprofiles,          &! in
           & addinterp,          &! in
           & profiles,           &! in
           & coef,               &! in
           & coef_scatt_ir,      &! in
           & optp,               &! in
           & addsun,             &
           & addaerosl,          &! in
           & addclouds,          &! in
           & calcemis_bf,        &! in
           & emissivity_bf,      &! inout
           & transmission,       &! out
           & radiancedata        &
#ifdef _RTTOV_TSTRAD_TEMP         
           & , temp              & 
#endif
     )

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

     If( emissivity_bf(j) == 0._JPRB ) Then
        xktem(j) = 0._JPRB
     Else
        xktem(j)=( radiancedata%bt(j) - radiance_fwd%bt(j) )/ inc_val
     End If

  End Do




  ixkdem(:)=Nint(xktem(:)*facem)

  !          ... and print it.
  Write (ioout,*)' '
  Write (ioout,*)'k-matrix: brute force '

  Write (ioout,*)' '
  Do j = 1 , nprint                   ! lwp on
     Write (ioout,'(a30)')title(j)
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Do i = 1 , nlev
        Write (ioout,333)i,(ixkdav(i,j,jch),jch=1,nchan_out)
     Enddo
     Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
     Write (ioout,*)' '
  Enddo

  Write (ioout,*)' surface variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = 1 , jpnsav
     Write (ioout,333)i,(ixkdsav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' skin variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = jpnsav+1 , jpnsav+jpnssv
     Write (ioout,333)i-jpnsav,(ixkdsav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  Write (ioout,*)' cloud variables '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do i = jpnsav+jpnssv+1 , sscvar
     Write (ioout,333)i-jpnsav-jpnssv,(ixkdsav(i,jch),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '
  Write (ioout,*)' surface emissivity '
  Write (ioout,3333)coef%ff_ori_chn(channels(1:nchan_out))
  Write (ioout,*)' '
  Do jp = 1, nprofiles
     joff = (jp-1) * nchan_out
     Write (ioout,333)jp,(ixkdem(jch+joff),jch=1,nchan_out)
  Enddo
  Write (ioout,*)' '

  !--------- CHECK CONSISTENCY BETWEEN TL AND BRUTE FORCE------------
  ! run TL with increments on all variables
  !
  emissivity(:)    = input_emissivity(:)
  emissivity_tl(:) = emissivity_inc(:)
  !
  ! if micro waves and  "calcemis" then mask out FASTEM2 (input <0)
  If ((coef % id_sensor == sensor_id_mw .or.coef % id_sensor == sensor_id_po) & 
     & .And. coef % fastem_ver >= 2 ) Then 
     !Where (input_emissivity(:) < 0.0)
     Where ( calcemis )
        emissivity_tl(:) = 0._JPRB
     End Where
  Endif

! call-4 of rttov_tl
!  write(6,*) 'call rttov_tl     --- to check consistency ( 1 call)'

  Call rttov_tl( &
        & rttov_errorstatus,  &! out
        & nprofiles,       &! in
        & nchannels,       &! in
        & channels,        &! in
        & lprofiles,       &! in
        & addinterp,       &! in
        & profiles,        &! in
        & prof_inc,        &! in
        & coef,            &! in
        & coef_scatt_ir,   &
        & optp,            &
        & addsun,          &
        & addaerosl,       &
        & addclouds,       &
        & calcemis,        &! in
        & emissivity,      &! inout
        & emissivity_tl,   &! inout
        & transmission,    &! inout
        & transmission_tl, &! inout
        & radiancedata,    &! inout
        & radiancedata_tl  &! inout 
#ifdef _RTTOV_TSTRAD_TEMP
        & , temp, temp_tl  &
#endif
        & ) 

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

  Do prof = 1, nprofiles
     sumrr = 0._JPRB
     Do jch=1,nchannels
        If(lprofiles(jch) == prof) Then
           sumrr = sumrr + radiancedata_tl % bt(jch)
        Endif
     End Do
     Write (IOOUT,149)prof,SUMRR
149  Format (1X,'PROFILE NUMBER=',I2,'  TL=',E20.10)
  End Do

  Do  ii =1,15
     fac = 10.0_JPRB**(ii-1)

     ! initialise profile with input profile
     Do jp = 1, nprofiles
        profiles_bf(jp) % p(:)   = profiles(jp) % p(:) + prof_inc(jp) % p(:) / fac
        profiles_bf(jp) % t(:)   = profiles(jp) % t(:) + prof_inc(jp) % t(:) / fac
        profiles_bf(jp) % q(:)   = profiles(jp) % q(:) + prof_inc(jp) %  q(:)  / fac
        profiles_bf(jp) % o3(:)  = profiles(jp) % o3(:) + prof_inc(jp) % o3(:)  / fac
        profiles_bf(jp) % co2(:) = profiles(jp) % co2(:) + prof_inc(jp) % co2(:)  / fac
        profiles_bf(jp) % co(:)  = profiles(jp) % co(:) + prof_inc(jp) % co(:)  / fac
        profiles_bf(jp) % n2o(:) = profiles(jp) % n2o(:) + prof_inc(jp) % n2o(:)  / fac
        profiles_bf(jp) % ch4(:) = profiles(jp) % ch4(:) + prof_inc(jp) % ch4(:)  / fac
        if(addaerosl)then
          profiles_bf(jp) % aerosols(:,:) = profiles(jp) % aerosols(:,:) + prof_inc(jp) % aerosols(:,:)  / fac
        endif
        if(addclouds)then
          profiles_bf(jp) % cloud(:,:) = profiles(jp) % cloud(:,:) + prof_inc(jp) % cloud(:,:)  / fac
          profiles_bf(jp) % cfrac(:,:) = profiles(jp) % cfrac(:,:) + prof_inc(jp) % cfrac(:,:)  / fac
        endif
        profiles_bf(jp) % clw(:) = profiles(jp) % clw(:) + prof_inc(jp) %  clw(:) / fac
        profiles_bf(jp) % s2m    = profiles(jp) % s2m 
        profiles_bf(jp) % s2m % t    = profiles(jp) % s2m % t  + prof_inc(jp) % s2m % t / fac
        profiles_bf(jp) % s2m % q   = profiles(jp) % s2m % q  + prof_inc(jp) % s2m % q / fac
        profiles_bf(jp) % s2m % p   = profiles(jp) % s2m % p  + prof_inc(jp) % s2m % p / fac
        profiles_bf(jp) % s2m % u   = profiles(jp) % s2m % u  + prof_inc(jp) % s2m % u / fac
        profiles_bf(jp) % s2m % v   = profiles(jp) % s2m % v  + prof_inc(jp) % s2m % v / fac
        profiles_bf(jp) % s2m % wfetc   = profiles(jp) % s2m % wfetc  + prof_inc(jp) % s2m % wfetc / fac
        profiles_bf(jp) % skin   = profiles(jp) % skin
        profiles_bf(jp) % skin % t   = profiles(jp) % skin % t + prof_inc(jp) % skin % t / fac
        profiles_bf(jp) % skin % fastem  = profiles(jp) % skin % fastem + prof_inc(jp) % skin % fastem / fac
        profiles_bf(jp) % ctp    = profiles(jp) % ctp + prof_inc(jp) % ctp  / fac
        profiles_bf(jp) % cfraction   = profiles(jp) % cfraction + prof_inc(jp) % cfraction / fac
        profiles_bf(jp) % ozone_Data  = profiles(jp) % ozone_Data
        profiles_bf(jp) % co2_Data    = profiles(jp) % co2_Data
        profiles_bf(jp) % co_Data    = profiles(jp) % co_Data
        profiles_bf(jp) % n2o_Data    = profiles(jp) % n2o_Data
        profiles_bf(jp) % ch4_Data    = profiles(jp) % ch4_Data
        if(addaerosl)then
          profiles_bf(jp) % aer_Data    = profiles(jp) % aer_Data
        endif
        if(addclouds)then
          profiles_bf(jp) % cld_Data    = profiles(jp) % cld_Data
        endif
        profiles_bf(jp) % clw_Data    = profiles(jp) % clw_Data
        profiles_bf(jp) % addsolar    = profiles(jp) % addsolar
        profiles_bf(jp) % addrefrac   = profiles(jp) % addrefrac
        profiles_bf(jp) % zenangle    = profiles(jp) % zenangle
        profiles_bf(jp) % azangle     = profiles(jp) % azangle
        profiles_bf(jp) % sunzenangle = profiles(jp) % sunzenangle
        profiles_bf(jp) % sunazangle  = profiles(jp) % sunazangle
        profiles_bf(jp) % latitude    = profiles(jp) % latitude
        profiles_bf(jp) % elevation   = profiles(jp) % elevation
        profiles_bf(jp) % Be          = profiles(jp) % Be
        profiles_bf(jp) % cosbk       = profiles(jp) % cosbk
     End Do

     ! use emissivity from FWD model output
     emissivity_bf(:) = emissivity_fwd(:) + emissivity_inc(:) / fac
     calcemis_bf(:)   = .False.

     ! if micro waves then mask out FASTEM2
  If ((coef % id_sensor == sensor_id_mw .or.coef % id_sensor == sensor_id_po) & 
     & .And. coef % fastem_ver >= 2 ) Then 
        !Where (input_emissivity(:) < 0.0)
        Where ( calcemis )
           emissivity_bf(:) = input_emissivity(:)
           calcemis_bf(:)   = calcemis(:)
        End Where
     Endif

! call-4 of rttov_direct
!     if(ii == 1) write(6,*) 'call rttov_direct --- to check consistency (15 calls)'
     !! ---  ii = ', ii, '  of 15'

     Call rttov_direct( &
           & rttov_errorstatus,  &! out
           & nprofiles,          &! in
           & nchannels,          &! in
           & channels,           &! in
           & lprofiles,          &! in
           & addinterp,          &! in
           & profiles_bf,        &! in
           & coef,               &! in
           & coef_scatt_ir,      &! in
           & optp,               &! in
           & addsun,             &
           & addaerosl,          &! in
           & addclouds,          &! in
           & calcemis_bf,        &! in
           & emissivity_bf,      &! inout
           & transmission,       &! out
           & radiancedata        &
#ifdef _RTTOV_TSTRAD_TEMP         
           & , temp              & 
#endif
        )

        If( any( rttov_errorstatus == errorstatus_fatal ) ) then
           Do jp = 1, nprofiles
              If( rttov_errorstatus(jp) == errorstatus_fatal ) then
                 Write( errMessage, '( "fatal error in rttov_tl")' )
                 Call Rttov_ErrorReport (rttov_errorstatus(jp), errMessage, NameOfRoutine)
              End If
           End Do
           Stop
        End If
        If( any( rttov_errorstatus /= errorstatus_success ) ) then
          Write( errMessage, '( "warning in rttov_tl")' )           
        End If

     sumr = 0._JPRB
     prof = 1  ! tests only first profile
     Do jch=1, nchannels
        If(lprofiles(jch) == prof) Then
           sumr = sumr + radiancedata%bt(jch) - radiance_fwd%bt(jch)
        End If
     End Do
     sumr = sumr*fac
     DIFFR=SUMR/SUMRR
     Write (IOOUT,148) SUMR,DIFFR,II
148  Format (1X,'BRUTE FORCE:   ',2E20.10,I10)

  End Do


#ifdef _RTTOV_TSTRAD_TEMP
  call rttov_alloc_temp(                         &
      & temp_status,                             &
      & nprofiles,                               &
      & nchannels,                               &
      & channels,                                &
      & addinterp,                               &
      & profiles(1)%nlevels,                     &
      & coef,                                    &
      & coef_scatt_ir,                           &
      & optp,                                    &
      & addsun .or. Any( profiles(:)%addsolar ), &
      & addaerosl,                               &
      & addclouds,                               &
      & 0_jpim,                                  &
      & temp = temp, temp_tl = temp_tl )     
  If( temp_status .ne. errorstatus_success ) Then
    Write( errMessage, '( "rttov_alloc_temp fatal error ")' )
    Call Rttov_ErrorReport( errorstatus_fatal, errMessage, NameOfRoutine )
    Stop       
  EndIf     
#endif

  Call rttov_alloc_prof( alloc_status(1), nprofiles, prof_inc, profiles(1) % nlevels, coef_scatt_ir, 0_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds )
  Call rttov_alloc_prof( alloc_status(2), nprofiles, null_inc, profiles(1) % nlevels, coef_scatt_ir, 0_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds )
  Call rttov_alloc_prof( alloc_status(3), nprofiles, profiles_tl, profiles(1) % nlevels, coef_scatt_ir, 0_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds )
  Call rttov_alloc_prof( alloc_status(4), nprofiles, profiles_bf, profiles(1) % nlevels, coef_scatt_ir, 0_jpim, &
    & addaerosl = addaerosl, addclouds = addclouds )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem deallocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  ! deallocate transmittances
  Deallocate( transmission % tau_total   ,stat= alloc_status(9))
  Deallocate( transmission % tau_layers  ,stat= alloc_status(10))
  Deallocate( transmission_tl % tau_total   ,stat= alloc_status(11))
  Deallocate( transmission_tl % tau_layers  ,stat= alloc_status(12))
  !
  ! deallocate radiance results arrays with number of channels

  Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata, profiles(1)%nlevels, 0_jpim )
  Call rttov_alloc_rad( alloc_status(1), nchannels, radiancedata_tl, profiles(1)%nlevels, 0_jpim )
  Call rttov_alloc_rad( alloc_status(1), nchannels, radiance_fwd, profiles(1)%nlevels, 0_jpim )
  If( any(alloc_status /= 0) ) then
     errorstatus = errorstatus_fatal
     Write( errMessage, '( "mem deallocation error")' )
     Call Rttov_ErrorReport (errorstatus, errMessage, NameOfRoutine)
IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',1_jpim,ZHOOK_HANDLE)
     Return
  End If

  Print *, ' tangent linear model test finished'
IF (LHOOK) CALL DR_HOOK('TSTRAD_TL',1_jpim,ZHOOK_HANDLE)
222 Format(1x,10f8.2)
333 Format(1x,i3,20i5)
444 Format(1x,10e8.2)
3333 Format(4x,20i5)
4444 Format(1x,10f8.3)

End Subroutine tstrad_tl
